@@ -30,7 +30,11 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
+#ifndef IS_MINGW
+#ifndef IS_SOLARIS
 #include <endian.h>
+#endif
+#endif
 #include <sys/stat.h>
 
 const char *plugin_version="LiVES asf/wmv decoder version 1.0";
@@ -253,7 +257,9 @@ static boolean get_tag(const lives_clip_data_t *cdata, AVFormatContext *s, const
     fprintf(stderr, "asf_decoder: Unsupported value type %d len %d in tag %s.\n", type, len, key);
     return TRUE;
   }
+#ifndef IS_MINGW
   av_metadata_set2(&s->metadata, key, value, 0);
+#endif
   free(value);
   return TRUE;
 }
@@ -865,6 +871,10 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
     return FALSE;
   }
 
+#ifdef IS_MINGW
+  setmode(priv->fd,O_BINARY);
+#endif
+
   if (read (priv->fd, header, ASF_PROBE_SIZE) < ASF_PROBE_SIZE) {
     // for example, might be a directory
 #ifdef DEBUG
@@ -898,7 +908,7 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
   cdata->arate=0;
   cdata->achans=0;
   cdata->asamps=0;
-  sprintf(cdata->audio_name,"none");
+  sprintf(cdata->audio_name,"%s","");
 
   priv->idx=NULL;
 
@@ -1208,7 +1218,6 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
 
       if (type == LIVES_MEDIA_TYPE_AUDIO) {
 	priv->st->codec->codec_type = AVMEDIA_TYPE_AUDIO;
-	snprintf(cdata->audio_name,16,"unknown");
 
       } else if (type == LIVES_MEDIA_TYPE_VIDEO) {
 	priv->st->codec->codec_type = AVMEDIA_TYPE_VIDEO;
@@ -1321,6 +1330,7 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
 	  if (priv->st->codec->extradata_size && (priv->st->codec->bits_per_coded_sample <= 8)) {
 	    priv->st->codec->palctrl = av_mallocz(sizeof(AVPaletteControl));
 
+#ifndef IS_MINGW
 # if __BYTE_ORDER == __BIG_ENDIAN
 	    for (i = 0; i < FFMIN(priv->st->codec->extradata_size, AVPALETTE_SIZE)/4; i++)
 	      
@@ -1329,6 +1339,11 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
 	    memcpy(priv->st->codec->palctrl->palette, priv->st->codec->extradata,
 		   FFMIN(priv->st->codec->extradata_size, AVPALETTE_SIZE));
 #endif
+#else
+	    memcpy(priv->st->codec->palctrl->palette, priv->st->codec->extradata,
+		   FFMIN(priv->st->codec->extradata_size, AVPALETTE_SIZE));
+#endif
+
 	    priv->st->codec->palctrl->palette_changed = 1;
 	  }
 	  
@@ -2137,6 +2152,9 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
     if (fps!=1000.) cdata->fps=fps;
   }
 
+
+#ifndef IS_MINGW
+
   if (cdata->fps==0.||cdata->fps==1000.) {
     // use mplayer to get fps if we can...it seems to have some magical way
     char cmd[1024];
@@ -2164,6 +2182,8 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
     }
   }
 
+#endif
+
   if (cdata->fps==0.&&ctx->time_base.num==0) {
     if (ctx->time_base.den==1) cdata->fps=25.;
   }
@@ -2203,7 +2223,9 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
 
 
 const char *module_check_init(void) {
+#ifndef IS_MINGW
   avcodec_init();
+#endif
   avcodec_register_all();
   return NULL;
 }
@@ -2230,6 +2252,8 @@ static lives_clip_data_t *init_cdata (void) {
 
   cdata->palettes=NULL;
   
+  cdata->sync_hint=0;
+
   return cdata;
 }
 
