@@ -58,6 +58,7 @@ const char *plugin_version="LiVES flv decoder version 1.0";
 
 #include <libavformat/avformat.h>
 #include <libavutil/avstring.h>
+#include <libavcodec/avcodec.h>
 
 #include "decplugin.h"
 #include "flv_decoder.h"
@@ -541,6 +542,10 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
     return FALSE;
   }
 
+#ifdef IS_MINGW
+  setmode(priv->fd,O_BINARY);
+#endif
+
   if (read (priv->fd, header, FLV_PROBE_SIZE) < FLV_PROBE_SIZE) {
     // for example, might be a directory
 #ifdef DEBUG
@@ -751,7 +756,7 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
 
   priv->ctx=ctx = avcodec_alloc_context();
 
-  sprintf(cdata->audio_name,"none");
+  sprintf(cdata->audio_name,"%s","");
 
   // now we get the stream data
   while (!got_astream || !got_vstream) {
@@ -804,7 +809,7 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
       case 11:
 	sprintf(cdata->audio_name,"%s","speex"); break;
       default:
-	sprintf(cdata->audio_name,"%s","unknown"); break;
+	break;
       }
 
       if (cdata->arate==0) {
@@ -1124,6 +1129,7 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
     if (fps!=1000.) cdata->fps=fps;
   }
 
+#ifndef IS_MINGW
   if (cdata->fps==0.||cdata->fps==1000.) {
     // use mplayer to get fps if we can...it seems to have some magical way
     char cmd[1024];
@@ -1150,6 +1156,7 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
       unlink(tmpfname);
     }
   }
+#endif
 
   if (cdata->fps==0.&&ctx->time_base.num==0) {
     // not sure about this
@@ -1202,7 +1209,9 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
 
 
 const char *module_check_init(void) {
+#ifndef IS_MINGW
   avcodec_init();
+#endif
   avcodec_register_all();
   return NULL;
 }
