@@ -69,13 +69,11 @@ const char *plugin_version="LiVES mkv decoder version 1.2";
 
 #include "mkv_decoder.h"
 
-//#include "mkv_decoder.h"
+#include <libavutil/intfloat_readwrite.h>
+#include <libavutil/intreadwrite.h>
+#include <libavutil/lzo.h>
+#include <libavutil/dict.h>
 
-#include "libavutil/intfloat_readwrite.h"
-#include "libavutil/intreadwrite.h"
-#include "libavutil/avstring.h"
-#include "libavutil/lzo.h"
-#include "libavutil/dict.h"
 #if CONFIG_ZLIB
 #include <zlib.h>
 #endif
@@ -1373,7 +1371,6 @@ static int lives_mkv_read_header(lives_clip_data_t *cdata) {
 
     } else if (track->type == MATROSKA_TRACK_TYPE_AUDIO) {
       priv->has_audio=TRUE;
-      sprintf(cdata->audio_name,"unknown");
     }
     if (encodings_list->nb_elem > 1) {
       fprintf(stderr,"mkv_decoder: Multiple combined encodings not supported\n");
@@ -1649,13 +1646,14 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
   AVCodec *codec=NULL;
   AVCodecContext *ctx;
 
+  int err;
+
   boolean got_picture=FALSE;
 
   struct stat sb;
-
   //#define DEBUG
 #ifdef DEBUG
-  fprintf(stderr,"\n");
+  fprintf(stderr,"\n\n\n\nDEBUG MKV");
 #endif
 
   priv->has_audio=priv->has_video=FALSE;
@@ -1667,10 +1665,15 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
     return FALSE;
   }
 
-  if (read (priv->fd, header, MKV_PROBE_SIZE) < MKV_PROBE_SIZE) {
+#ifdef IS_MINGW
+  setmode(priv->fd,O_BINARY);
+#endif
+
+  if ((err=read (priv->fd, header, MKV_PROBE_SIZE)) < MKV_PROBE_SIZE) {
     // for example, might be a directory
 #ifdef DEBUG
-    fprintf(stderr, "mkv_decoder: unable to read header for %s\n",cdata->URI);
+    fprintf(stderr, "mkv_decoder: unable to read header %d %d for %s\n",err,MKV_PROBE_SIZE,cdata->URI);
+    if (err==0)  fprintf(stderr,"err was %s %d\n",strerror(errno),priv->filesize);
 #endif
     close(priv->fd);
     return FALSE;
@@ -1700,7 +1703,10 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
   priv->idxhh=NULL;
   priv->idxht=NULL;
 
-  sprintf(cdata->audio_name,"none");
+  fstat(priv->fd,&sb);
+  priv->filesize=sb.st_size;
+
+  sprintf(cdata->audio_name,"%s","");
 
   priv->s = avformat_alloc_context();
 
@@ -1712,15 +1718,11 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
   priv->avpkt.data=NULL;
   priv->ctx=NULL;
 
-  fstat(priv->fd,&sb);
-  priv->filesize=sb.st_size;
-
   if (lives_mkv_read_header(cdata)) {
     close(priv->fd);
     return FALSE;
   }
 
-
   cdata->seek_flag=LIVES_SEEK_FAST|LIVES_SEEK_NEEDS_CALCULATION;
 
   cdata->offs_x=0;
@@ -1749,7 +1751,7 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
   priv->ctx = ctx = avcodec_alloc_context();
 
   if (avcodec_open(ctx, codec) < 0) {
-    fprintf(stderr, "mkv_decoder: Could not open avcodec context\n");
+    fprintf(stderr, "mkv_decoder: Could not open avcodec context for codec\n");
     detach_stream(cdata);
     return FALSE;
   }
@@ -1764,7 +1766,6 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
 
   matroska_read_seek(cdata,0);
 
-
   while (!got_picture&&!got_eof) {
     matroska_read_packet(cdata,&priv->avpkt);
     
@@ -1804,6 +1805,12 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
   }
   else pts=dts=0;
 
+  if (priv->avpkt.data!=NULL) {
+    free(priv->avpkt.data);
+    priv->avpkt.data=NULL;
+    priv->avpkt.size=0;
+  }
+
   cdata->YUV_clamping=WEED_YUV_CLAMPING_UNCLAMPED;
   if (ctx->color_range==AVCOL_RANGE_MPEG) cdata->YUV_clamping=WEED_YUV_CLAMPING_CLAMPED;
 
@@ -1861,27 +1868,34 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
     // use mplayer to get fps if we can...it seems to have some magical way
     char cmd[1024];
     char tmpfname[32];
-    int ofd;
+    int res;
 
-    sprintf(tmpfname,"/tmp/mkvdec=XXXXXX");
-    ofd=mkstemp(tmpfname);
-    if (ofd!=-1) {
-      int res;
-      snprintf(cmd,1024,"LANGUAGE=en LANG=en mplayer \"%s\" -identify -frames 0 2>/dev/null | grep ID_VIDEO_FPS > %s",cdata->URI,tmpfname);
+    sprintf(tmpfname,"mkvdec=XXXXXX");
+#ifndef IS_MINGW
+    snprintf(cmd,1024,"LANGUAGE=en LANG=en mplayer \"%s\" -identify -frames 0 2>/dev/null | grep ID_VIDEO_FPS > %s",cdata->URI,tmpfname);
+#else
+    snprintf(cmd,1024,"mplayer.exe \"%s\" -identify -frames 0 2>NUL | grep.exe ID_VIDEO_FPS > %s",cdata->URI,tmpfname);
+#endif 
+    res=system(cmd);
       
-      res=system(cmd);
-      
-      if (!res) {
-	char buffer[1024];
-	ssize_t bytes=read(ofd,buffer,1024);
+    if (!res) {
+      char buffer[1024];
+      ssize_t bytes;
+      int ofd=open(tmpfname,O_RDONLY);
+      if (ofd>-1) {
+
+#ifdef IS_MINGW
+	setmode(ofd,O_BINARY);
+#endif
+	bytes=read(ofd,buffer,1024);
 	memset(buffer+bytes,0,1);
 	if (!(strncmp(buffer,"ID_VIDEO_FPS=",13))) {
 	  cdata->fps=strtod (buffer+13,NULL);
 	}
+	close(ofd);
       }
-      close(ofd);
-      unlink(tmpfname);
     }
+    unlink(tmpfname);
   }
 
   if (cdata->fps==0.||cdata->fps==1000.) {
@@ -1941,7 +1955,9 @@ static boolean attach_stream(lives_clip_data_t *cdata) {
 
 
 const char *module_check_init(void) {
+#ifndef IS_MINGW
   avcodec_init();
+#endif
   avcodec_register_all();
   av_log_set_level(AV_LOG_ERROR);
   return NULL;
@@ -1976,6 +1992,8 @@ static lives_clip_data_t *init_cdata (void) {
 
   cdata->nframes=0;
 
+  cdata->sync_hint=0;
+
   got_eof=FALSE;
   errval=0;
   
@@ -2459,8 +2477,10 @@ static index_entry * matroska_read_seek(const lives_clip_data_t *cdata,
 
   if (!priv->idxhh) return NULL;
 
-  timestamp = FFMIN(timestamp, frame_to_dts(cdata,cdata->nframes));
-  timestamp = FFMAX(timestamp, priv->idxhh->dts);
+  if (timestamp!=0) {
+    timestamp = FFMIN(timestamp, frame_to_dts(cdata,cdata->nframes));
+    timestamp = FFMAX(timestamp, priv->idxhh->dts);
+  }
 
   idx=get_idx_for_pts(cdata,timestamp);
 
@@ -2481,7 +2501,7 @@ static index_entry * matroska_read_seek(const lives_clip_data_t *cdata,
   matroska->skip_to_timecode = idx->dts;
   matroska->done = 0;
 
-  ff_update_cur_dts(s, st, idx->dts);
+  //ff_update_cur_dts(s, st, idx->dts);
 
   return idx;
 }
