@@ -192,7 +192,7 @@ const int *get_audio_fmts() {
 const char rfx[32768];
 
 
-const char *get_rfx (void) {
+const char *get_init_rfx (void) {
   char **vdevs = get_vloopback2_devices();
   char devstr[30000];
   size_t slen=0;
@@ -390,7 +390,7 @@ boolean init_screen (int width, int height, boolean fullscreen, uint64_t window_
 
 
 
-boolean render_frame (int hsize, int vsize, int64_t tc, void **pixel_data, void **return_data) {
+boolean render_frame (int hsize, int vsize, int64_t tc, void **pixel_data, void **rd, void **pp) {
   // hsize and vsize are in [macro]pixels (n-byte)
   size_t frame_size,bytes;
 
