@@ -332,12 +332,18 @@ int scribbler_process (weed_plant_t *inst, weed_timecode_t timestamp) {
 }
 
 
-inline static int font_compare(const void *p1, const void *p2) {  
+static int font_compare(const void *p1, const void *p2) {  
   const char *s1 = (const char *)(*(char **)p1);
   const char *s2 = (const char *)(*(char **)p2);
-  return(strcasecmp(s1, s2));
+  char *u1 = g_utf8_casefold(s1, -1);
+  char *u2 = g_utf8_casefold(s2, -1);
+  int ret = strcmp(u1, u2);
+  g_free(u1);
+  g_free(u2);
+  return ret;
 }
 
+
 weed_plant_t *weed_setup (weed_bootstrap_f weed_boot) {
   weed_plant_t *plugin_info=weed_plugin_info_init(weed_boot,num_versions,api_versions);
   weed_plant_t **clone1,**clone2;
@@ -403,8 +409,8 @@ weed_plant_t *weed_setup (weed_bootstrap_f weed_boot) {
     in_params[P_FGALPHA]=weed_float_init("fr_alpha","_Alpha _Foreground",1.0,0.0,1.0);
     in_params[P_BGALPHA]=weed_float_init("bg_alpha","_Alpha _Background",1.0,0.0,1.0);
     in_params[P_FONTSIZE]=weed_float_init("fontsize","_Font Size",20.0,10.0,128.0);
-    in_params[P_CENTER]=weed_switch_init("center","_Center text",1);
-    in_params[P_RISE]=weed_switch_init("rising","_Rising text",1);
+    in_params[P_CENTER]=weed_switch_init("center","_Center text",WEED_TRUE);
+    in_params[P_RISE]=weed_switch_init("rising","_Rising text",WEED_TRUE);
     in_params[P_TOP]=weed_float_init("top","_Top",0.0,0.0,1.0);
     in_params[P_END]=NULL;
     
