@@ -1,6 +1,6 @@
 // audio.c
 // LiVES (lives-exe)
-// (c) G. Finch 2005 - 2011
+// (c) G. Finch 2005 - 2012
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -17,10 +17,12 @@
 #ifdef HAVE_SYSTEM_WEED
 #include <weed/weed.h>
 #include <weed/weed-host.h>
+#include <weed/weed-effects.h>
 #include <weed/weed-palettes.h>
 #else
 #include "../libweed/weed.h"
 #include "../libweed/weed-host.h"
+#include "../libweed/weed-effects.h"
 #include "../libweed/weed-palettes.h"
 #endif
 
@@ -62,13 +64,13 @@ void audio_free_fnames(void) {
 
 
 
-LIVES_INLINE void sample_silence_dS (float *dst, unsigned long nsamples) {
+LIVES_INLINE void sample_silence_dS (float *dst, uint64_t nsamples) {
   memset(dst,0,nsamples*sizeof(float));
 }
 
 
 void sample_move_d8_d16(short *dst, unsigned char *src,
-			unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_sign) {
+			uint64_t nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_sign) {
   // convert 8 bit audio to 16 bit audio
 
   register int nSrcCount, nDstCount;
@@ -123,7 +125,7 @@ void sample_move_d8_d16(short *dst, unsigned char *src,
 
 /* convert from any number of source channels to any number of destination channels */
 void sample_move_d16_d16(short *dst, short *src,
-			 unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_endian, int swap_sign) {
+			 uint64_t nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_endian, int swap_sign) {
   register int nSrcCount, nDstCount;
   register float src_offset_f=0.f;
   register int src_offset_i=0;
@@ -198,7 +200,7 @@ void sample_move_d16_d16(short *dst, short *src,
 
 /* convert from any number of source channels to any number of destination channels - 8 bit output */
 void sample_move_d16_d8(uint8_t *dst, short *src,
-			 unsigned long nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_sign) {
+			 uint64_t nsamples, size_t tbytes, float scale, int nDstChannels, int nSrcChannels, int swap_sign) {
   register int nSrcCount, nDstCount;
   register float src_offset_f=0.f;
   register int src_offset_i=0;
@@ -250,7 +252,7 @@ void sample_move_d16_d8(uint8_t *dst, short *src,
 
 
 
-void sample_move_d16_float (float *dst, short *src, unsigned long nsamples, unsigned long src_skip, int is_unsigned, float vol) {
+void sample_move_d16_float (float *dst, short *src, uint64_t nsamples, uint64_t src_skip, int is_unsigned, float vol) {
   // convert 16 bit audio to float audio
 
 
@@ -303,15 +305,22 @@ void sample_move_d16_float (float *dst, short *src, unsigned long nsamples, unsi
 
 
 
+void sample_move_float_float (float *dst, float *src, uint64_t nsamples, uint64_t src_skip, float vol) {
+  // copy one channel of (interleaved) float to a buffer, applying the volume
+  register int i;
+
+  for (i=0;i<nsamples;i++) {
+    *(dst++)=*src*vol;
+    src+=src_skip;
+  }
+}
 
 
 
-
-
-
-long sample_move_float_int(void *holding_buff, float **float_buffer, int nsamps, float scale, int chans, int asamps, int usigned, gboolean swap_endian, float vol) {
+int64_t sample_move_float_int(void *holding_buff, float **float_buffer, int nsamps, float scale, int chans, int asamps, 
+			      int usigned, gboolean little_endian, gboolean interleaved, float vol) {
   // convert float samples back to int
-  long frames_out=0l;
+  int64_t frames_out=0l;
   register int i;
   register int offs=0,coffs=0;
   register float coffs_f=0.f;
@@ -323,17 +332,18 @@ long sample_move_float_int(void *holding_buff, float **float_buffer, int nsamps,
   register unsigned short valu=0;
   register float valf;
 
+
   while ((nsamps-coffs)>0) {
     frames_out++;
     for (i=0;i<chans;i++) {
-      valf=*(float_buffer[i]+coffs);
+      valf=*(float_buffer[i]+(interleaved?(coffs*chans):coffs));
       valf*=vol;
       val=(short)(valf*(valf>0.?SAMPLE_MAX_16BIT_P:SAMPLE_MAX_16BIT_N));
 
       if (usigned) valu=(val+SAMPLE_MAX_16BITI);
 
       if (asamps==16) {
-	if (!swap_endian) {
+	if (!little_endian) {
 	  if (usigned) *(hbuffu+offs)=valu;
 	  else *(hbuffs+offs)=val;
 	}
@@ -363,7 +373,7 @@ long sample_move_float_int(void *holding_buff, float **float_buffer, int nsamps,
 
 // play from memory buffer
 
-long sample_move_abuf_float (float **obuf, int nchans, int nsamps, int out_arate, float vol) {
+int64_t sample_move_abuf_float (float **obuf, int nchans, int nsamps, int out_arate, float vol) {
 
   int samples_out=0;
 
@@ -468,7 +478,7 @@ long sample_move_abuf_float (float **obuf, int nchans, int nsamps, int out_arate
 
 
 
-long sample_move_abuf_int16 (short *obuf, int nchans, int nsamps, int out_arate) {
+int64_t sample_move_abuf_int16 (short *obuf, int nchans, int nsamps, int out_arate) {
 
   int samples_out=0;
 
@@ -574,7 +584,7 @@ long sample_move_abuf_int16 (short *obuf, int nchans, int nsamps, int out_arate)
 
 
 
-/// copy a memory chunk into an audio buffer, applying overall volume control
+/// copy a memory chunk into an audio buffer
 
 
 static size_t chunk_to_float_abuf(lives_audio_buf_t *abuf, float **float_buffer, int nsamps) {
@@ -615,7 +625,7 @@ static size_t chunk_to_int16_abuf(lives_audio_buf_t *abuf, float **float_buffer,
 
 //#define DEBUG_ARENDER
 
-static gboolean pad_with_silence(int out_fd, off64_t oins_size, long ins_size, int asamps, int aunsigned, gboolean big_endian) {
+static gboolean pad_with_silence(int out_fd, off64_t oins_size, int64_t ins_size, int asamps, int aunsigned, gboolean big_endian) {
   // fill to ins_pt with zeros (or 0x80.. for unsigned)
   guchar *zero_buff;
   size_t sblocksize=SILENCE_BLOCK_SIZE;
@@ -664,7 +674,7 @@ static gboolean pad_with_silence(int out_fd, off64_t oins_size, long ins_size, i
 
 
 
-long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *avels, gdouble *fromtime, weed_timecode_t tc_start, weed_timecode_t tc_end, gdouble *chvol, gdouble opvol_start, gdouble opvol_end, lives_audio_buf_t *obuf) {
+int64_t render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *avels, gdouble *fromtime, weed_timecode_t tc_start, weed_timecode_t tc_end, gdouble *chvol, gdouble opvol_start, gdouble opvol_end, lives_audio_buf_t *obuf) {
   // called during multitrack rendering to create the actual audio file
   // (or in-memory buffer for preview playback in multitrack)
 
@@ -715,16 +725,16 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
   register int i,j;
   ssize_t bytes_read;
   int in_fd[nfiles],out_fd=-1;
-  gulong nframes;
+  uint64_t nframes;
   gboolean in_reverse_endian[nfiles],out_reverse_endian=FALSE;
   off64_t seekstart[nfiles];
   gchar *infilename,*outfilename;
   weed_timecode_t tc=tc_start;
   gdouble ins_pt=tc/U_SEC;
-  long ins_size=0l,cur_size;
+  int64_t ins_size=0l,cur_size;
   gdouble time=0.;
   gdouble opvol=opvol_start;
-  long frames_out=0;
+  int64_t frames_out=0;
 
   int track;
 
@@ -737,9 +747,9 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
   gboolean is_silent[nfiles];
   gint first_nonsilent=-1;
 
-  long tsamples=((tc_end-tc_start)/U_SEC*out_arate+.5);
+  int64_t tsamples=((tc_end-tc_start)/U_SEC*out_arate+.5);
 
-  long blocksize,zsamples,xsamples;
+  int64_t blocksize,zsamples,xsamples;
 
   void *finish_buff;
 
@@ -749,10 +759,10 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
   gint max_segments;
   gdouble zavel,zavel_max=0.;
 
-  long tot_frames=0l;
+  int64_t tot_frames=0l;
 
   int render_block_size=RENDER_BLOCK_SIZE;
-
+  
   if (out_achans*nfiles*tsamples==0) return 0l;
 
   if (!storedfdsset) audio_reset_stored_fnames();
@@ -775,10 +785,14 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
       return 0l;
     }
 
+#ifdef IS_MINGW
+    setmode(out_fd, O_BINARY);
+#endif
+
     cur_size=get_file_size(out_fd);
 
     ins_pt*=out_achans*out_arate*out_asamps;
-    ins_size=((long)(ins_pt/out_achans/out_asamps+.5))*out_achans*out_asamps;
+    ins_size=((int64_t)(ins_pt/out_achans/out_asamps+.5))*out_achans*out_asamps;
 
     if ((!out_bendian&&(capable->byte_order==LIVES_BIG_ENDIAN))||
 	(out_bendian&&(capable->byte_order==LIVES_LITTLE_ENDIAN))) 
@@ -856,6 +870,9 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 	  mainw->read_failed=TRUE;
 	}
 
+#ifdef IS_MINGW
+	setmode(in_fd[track], O_BINARY);
+#endif
 	if (track<NSTOREDFDS) {
 	  storedfds[track]=in_fd[track];
 	  storedfnames[track]=g_strdup(infilename);
@@ -879,10 +896,10 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
     // all in tracks are empty
     // output silence
     if (to_file>-1) {
-      long oins_size=ins_size;
+      int64_t oins_size=ins_size;
       ins_pt=tc_end/U_SEC;
       ins_pt*=out_achans*out_arate*out_asamps;
-      ins_size=((long)(ins_pt/out_achans/out_asamps)+.5)*out_achans*out_asamps;
+      ins_size=((int64_t)(ins_pt/out_achans/out_asamps)+.5)*out_achans*out_asamps;
       pad_with_silence(out_fd,oins_size,ins_size,out_asamps,out_unsigned,out_bendian);
       //sync();
       close (out_fd);
@@ -908,6 +925,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
     return tsamples;
   }
 
+
   // we don't want to use more than MAX_AUDIO_MEM bytes
   // (numbers will be much larger than examples given)
 
@@ -981,6 +999,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 
       if (from_files[track]==mainw->ascrap_file) {
 	// be forgiving with the ascrap file
+
 	if (mainw->read_failed) mainw->read_failed=FALSE;
       }
 
@@ -1045,7 +1064,7 @@ long render_audio_segment(gint nfiles, gint *from_files, gint to_file, gdouble *
 	// output to file
 	// convert back to int; use out_scale of 1., since we did our resampling in sample_move_*_d16
 	frames_out=sample_move_float_int((void *)finish_buff,chunk_float_buffer,blocksize,1.,out_achans,
-					 out_asamps*8,out_unsigned,out_reverse_endian,opvol);
+					 out_asamps*8,out_unsigned,out_reverse_endian,FALSE,opvol);
 	lives_write (out_fd,finish_buff,frames_out*out_asamps*out_achans,TRUE);
 #ifdef DEBUG_ARENDER
 	g_print(".");
@@ -1128,7 +1147,7 @@ void jack_rec_audio_to_clip(gint fileno, gint old_file, lives_rec_audio_type_t r
   file *outfile=mainw->files[fileno];
   gchar *outfilename=g_build_filename(prefs->tmpdir,outfile->handle,"audio",NULL);
   int retval;
-  
+
   do {
     retval=0;
     mainw->aud_rec_fd=open(outfilename,O_WRONLY|O_CREAT|O_APPEND,S_IRUSR|S_IWUSR);
@@ -1143,32 +1162,54 @@ void jack_rec_audio_to_clip(gint fileno, gint old_file, lives_rec_audio_type_t r
 
   g_free(outfilename);
 
-  mainw->jackd_read=jack_get_driver(0,FALSE);
-  mainw->jackd_read->playing_file=fileno;
-  mainw->jackd_read->frames_written=0;
+  if (rec_type==RECA_GENERATED) {
+    mainw->jackd->playing_file=fileno;
+  }
+  else {
+    mainw->jackd_read=jack_get_driver(0,FALSE);
+    mainw->jackd_read->playing_file=fileno;
+    mainw->jackd_read->frames_written=0;
+  }
 
 
-  if (rec_type==RECA_EXTERNAL) {
+#ifdef IS_MINGW
+  setmode(mainw->aud_rec_fd, O_BINARY);
+#endif
+
+  if (rec_type==RECA_EXTERNAL||rec_type==RECA_GENERATED) {
     gint asigned;
     gint aendian;
-    gulong fsize=get_file_size(mainw->aud_rec_fd);
+    uint64_t fsize=get_file_size(mainw->aud_rec_fd);
 
-    mainw->jackd_read->reverse_endian=FALSE;
- 
-    // start jack recording
-    jack_open_device_read(mainw->jackd_read);
-    jack_read_driver_activate(mainw->jackd_read);
-
-    outfile->arate=outfile->arps=mainw->jackd_read->sample_in_rate;
-    outfile->achans=mainw->jackd_read->num_input_channels;
-
-    outfile->asampsize=16;
-    outfile->signed_endian=get_signed_endian(TRUE,TRUE);
+    if (rec_type==RECA_EXTERNAL) {
+      mainw->jackd_read->reverse_endian=FALSE;
     
-    asigned=!(outfile->signed_endian&AFORM_UNSIGNED);
-    aendian=!(outfile->signed_endian&AFORM_BIG_ENDIAN);
-
-    mainw->jackd_read->frames_written=fsize/(outfile->achans*(outfile->asampsize>>3));
+      // start jack recording
+      jack_open_device_read(mainw->jackd_read);
+      jack_read_driver_activate(mainw->jackd_read);
+      
+      outfile->arate=outfile->arps=mainw->jackd_read->sample_in_rate;
+      outfile->achans=mainw->jackd_read->num_input_channels;
+      
+      outfile->asampsize=16;
+      outfile->signed_endian=get_signed_endian(TRUE,TRUE);
+      
+      asigned=!(outfile->signed_endian&AFORM_UNSIGNED);
+      aendian=!(outfile->signed_endian&AFORM_BIG_ENDIAN);
+      
+      mainw->jackd_read->frames_written=fsize/(outfile->achans*(outfile->asampsize>>3));
+    }
+    else {
+      mainw->jackd->reverse_endian=FALSE;
+      outfile->arate=outfile->arps=mainw->jackd->sample_out_rate;
+      outfile->achans=mainw->jackd->num_output_channels;
+      
+      outfile->asampsize=16;
+      outfile->signed_endian=get_signed_endian(TRUE,TRUE);
+      
+      asigned=!(outfile->signed_endian&AFORM_UNSIGNED);
+      aendian=!(outfile->signed_endian&AFORM_BIG_ENDIAN);
+    }
 
     save_clip_value(fileno,CLIP_DETAILS_ACHANS,&outfile->achans);
     save_clip_value(fileno,CLIP_DETAILS_ARATE,&outfile->arps);
@@ -1192,7 +1233,7 @@ void jack_rec_audio_to_clip(gint fileno, gint old_file, lives_rec_audio_type_t r
   }
 
   // in grab window mode, just return, we will call rec_audio_end on playback end
-  if (rec_type==RECA_WINDOW_GRAB||rec_type==RECA_EXTERNAL) return;
+  if (rec_type==RECA_WINDOW_GRAB||rec_type==RECA_EXTERNAL||rec_type==RECA_GENERATED) return;
 
   mainw->cancelled=CANCEL_NONE;
   mainw->cancel_type=CANCEL_SOFT;
@@ -1218,7 +1259,7 @@ void jack_rec_audio_end(void) {
   // recording ended
 
   // stop recording
-  jack_close_device(mainw->jackd_read);
+  if (mainw->jackd_read!=NULL) jack_close_device(mainw->jackd_read);
   mainw->jackd_read=NULL;
 
   // close file
@@ -1251,29 +1292,51 @@ void pulse_rec_audio_to_clip(gint fileno, gint old_file, lives_rec_audio_type_t
     }
   } while (retval==LIVES_RETRY);
 
-  mainw->pulsed_read=pulse_get_driver(FALSE);
-  mainw->pulsed_read->playing_file=fileno;
-  mainw->pulsed_read->frames_written=0;
+  if (rec_type==RECA_GENERATED) {
+    mainw->pulsed->playing_file=fileno;
+  }
+  else {
+    mainw->pulsed_read=pulse_get_driver(FALSE);
+    mainw->pulsed_read->playing_file=fileno;
+    mainw->pulsed_read->frames_written=0;
+  }
 
-  if (rec_type==RECA_EXTERNAL) {
+#ifdef IS_MINGW
+  setmode(mainw->aud_rec_fd, O_BINARY);
+#endif
+
+  if (rec_type==RECA_EXTERNAL||rec_type==RECA_GENERATED) {
     gint asigned;
     gint aendian;
-    gulong fsize=get_file_size(mainw->aud_rec_fd);
+    uint64_t fsize=get_file_size(mainw->aud_rec_fd);
 
-    mainw->pulsed_read->reverse_endian=FALSE;
-
-    pulse_driver_activate(mainw->pulsed_read);
-
-    outfile->arate=outfile->arps=mainw->pulsed_read->out_arate;
-    outfile->achans=mainw->pulsed_read->out_achans;
-    outfile->asampsize=mainw->pulsed_read->out_asamps;
-    outfile->signed_endian=get_signed_endian(mainw->pulsed_read->out_signed!=AFORM_UNSIGNED,
-					     mainw->pulsed_read->out_endian!=AFORM_BIG_ENDIAN);
-    
-    asigned=!(outfile->signed_endian&AFORM_UNSIGNED);
-    aendian=!(outfile->signed_endian&AFORM_BIG_ENDIAN);
-
-    mainw->pulsed_read->frames_written=fsize/(outfile->achans*(outfile->asampsize>>3));
+    if (rec_type==RECA_EXTERNAL) {
+      mainw->pulsed_read->reverse_endian=FALSE;
+      
+      pulse_driver_activate(mainw->pulsed_read);
+      
+      outfile->arate=outfile->arps=mainw->pulsed_read->out_arate;
+      outfile->achans=mainw->pulsed_read->out_achans;
+      outfile->asampsize=mainw->pulsed_read->out_asamps;
+      outfile->signed_endian=get_signed_endian(mainw->pulsed_read->out_signed!=AFORM_UNSIGNED,
+					       mainw->pulsed_read->out_endian!=AFORM_BIG_ENDIAN);
+      
+      asigned=!(outfile->signed_endian&AFORM_UNSIGNED);
+      aendian=!(outfile->signed_endian&AFORM_BIG_ENDIAN);
+      
+      mainw->pulsed_read->frames_written=fsize/(outfile->achans*(outfile->asampsize>>3));
+    }
+    else {
+      mainw->pulsed->reverse_endian=FALSE;
+      outfile->arate=outfile->arps=mainw->pulsed->out_arate;
+      outfile->achans=mainw->pulsed->out_achans;
+      outfile->asampsize=mainw->pulsed->out_asamps;
+      outfile->signed_endian=get_signed_endian(mainw->pulsed->out_signed!=AFORM_UNSIGNED,
+					       mainw->pulsed->out_endian!=AFORM_BIG_ENDIAN);
+      
+      asigned=!(outfile->signed_endian&AFORM_UNSIGNED);
+      aendian=!(outfile->signed_endian&AFORM_BIG_ENDIAN);
+    }
 
     save_clip_value(fileno,CLIP_DETAILS_ACHANS,&outfile->achans);
     save_clip_value(fileno,CLIP_DETAILS_ARATE,&outfile->arps);
@@ -1296,7 +1359,7 @@ void pulse_rec_audio_to_clip(gint fileno, gint old_file, lives_rec_audio_type_t
   }
 
   // in grab window mode, just return, we will call rec_audio_end on playback end
-  if (rec_type==RECA_WINDOW_GRAB||rec_type==RECA_EXTERNAL) return;
+  if (rec_type==RECA_WINDOW_GRAB||rec_type==RECA_EXTERNAL||rec_type==RECA_GENERATED) return;
 
   mainw->cancelled=CANCEL_NONE;
   mainw->cancel_type=CANCEL_SOFT;
@@ -1321,12 +1384,14 @@ void pulse_rec_audio_end(void) {
 
   // stop recording
 
-  pa_threaded_mainloop_lock(mainw->pulsed->mloop);
-  pulse_flush_read_data(mainw->pulsed_read,0,NULL);
-  pulse_close_client(mainw->pulsed_read);
-  pa_threaded_mainloop_unlock(mainw->pulsed->mloop);
+  if (mainw->pulsed_read!=NULL) {
+    pa_threaded_mainloop_lock(mainw->pulsed->mloop);
+    pulse_flush_read_data(mainw->pulsed_read,mainw->pulsed_read->playing_file,0,mainw->pulsed->reverse_endian,NULL);
+    pulse_close_client(mainw->pulsed_read);
+    pa_threaded_mainloop_unlock(mainw->pulsed->mloop);
 
-  mainw->pulsed_read=NULL;
+    mainw->pulsed_read=NULL;
+  }
 
   // close file
   close(mainw->aud_rec_fd);
@@ -1393,6 +1458,7 @@ static lives_audio_track_state_t *aframe_to_atstate(weed_plant_t *event) {
 
   int btoffs=mainw->multitrack!=NULL?mainw->multitrack->opts.back_audio_tracks:1;
 
+
   for (i=0;i<num_aclips;i+=2) {
     if (aclips[i+1]>0) { // else ignore
       atrack=aclips[i];
@@ -1416,7 +1482,8 @@ static lives_audio_track_state_t *aframe_to_atstate(weed_plant_t *event) {
 
 
 
-lives_audio_track_state_t *get_audio_and_effects_state_at(weed_plant_t *event_list, weed_plant_t *st_event, gboolean get_audstate, gboolean exact) {
+lives_audio_track_state_t *get_audio_and_effects_state_at(weed_plant_t *event_list, weed_plant_t *st_event, 
+							  gboolean get_audstate, gboolean exact) {
 
   // if exact is set, we must rewind back to first active stateful effect, 
   // and play forwards from there (not yet implemented - TODO)
@@ -1485,7 +1552,6 @@ lives_audio_track_state_t *get_audio_and_effects_state_at(weed_plant_t *event_li
     nevent=get_next_event(event);
   } while (event!=st_event);
 
-
   if (audstate!=NULL) {
     for (nfiles=0;audstate[nfiles].afile!=-1;nfiles++);
 
@@ -1513,9 +1579,9 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
 
   // all we really do here is set from_files,aseeks and avels arrays and call render_audio_segment
 
-  lives_audio_track_state_t *atstate;
+  lives_audio_track_state_t *atstate=NULL;
   int nnfiles,i;
-  gdouble chvols[65536]; // TODO - use list
+  gdouble chvols[MAX_AUDIO_TRACKS]; // TODO - use list
 
   static weed_timecode_t last_tc;
   static weed_timecode_t fill_tc;
@@ -1533,7 +1599,6 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
 
   if (st_event!=NULL) {
     // this is only called for the first buffered read
-
     event=st_event;
     last_tc=get_event_timecode(event);
 
@@ -1564,7 +1629,12 @@ void fill_abuffer_from(lives_audio_buf_t *abuf, weed_plant_t *event_list, weed_p
     // start of the track buffer
 
 
-    atstate=get_audio_and_effects_state_at(event_list,event,TRUE,exact);
+    if (event!=get_first_event(event_list)) 
+      atstate=get_audio_and_effects_state_at(event_list,event,TRUE,exact);
+
+    // process audio updates at this frame
+    else atstate=aframe_to_atstate(event);
+
     
     if (atstate!=NULL) {
       
@@ -1779,11 +1849,12 @@ gboolean resync_audio(gint frameno) {
 
   // this is only active if "audio follows video rate/fps changes" is set
 
-  if (!(prefs->audio_opts&AUDIO_OPTS_FOLLOW_FPS)) return FALSE;
+    if (!(prefs->audio_opts&AUDIO_OPTS_FOLLOW_FPS)) return FALSE;
 
   // if recording external audio, we are intrinsically in sync
   if (mainw->record&&(prefs->rec_opts&REC_EXT_AUDIO)) return TRUE;
 
+
 #ifdef ENABLE_JACK
   if (prefs->audio_player==AUD_PLAYER_JACK&&mainw->jackd!=NULL) {
     if (!mainw->is_rendering) {
@@ -1792,10 +1863,13 @@ gboolean resync_audio(gint frameno) {
 	if (jack_try_reconnect()) jack_audio_seek_frame(mainw->jackd,frameno);
       }
 
-      mainw->rec_aclip=mainw->current_file;
-      mainw->rec_avel=cfile->pb_fps/cfile->fps;
-      mainw->rec_aseek=(gdouble)mainw->jackd->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+      if (mainw->agen_key==0&&!mainw->agen_needs_reinit) {
+	mainw->rec_aclip=mainw->current_file;
+	mainw->rec_avel=cfile->pb_fps/cfile->fps;
+	mainw->rec_aseek=(gdouble)mainw->jackd->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+      }
     }
+
     return TRUE;
   }
 #endif
@@ -1806,9 +1880,11 @@ gboolean resync_audio(gint frameno) {
       if (!pulse_audio_seek_frame(mainw->pulsed,frameno)) {
 	if (pulse_try_reconnect()) pulse_audio_seek_frame(mainw->pulsed,frameno);
       }
-      mainw->rec_aclip=mainw->current_file;
-      mainw->rec_avel=cfile->pb_fps/cfile->fps;
-      mainw->rec_aseek=(gdouble)mainw->pulsed->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+      if (mainw->agen_key==0&&!mainw->agen_needs_reinit) {
+	mainw->rec_aclip=mainw->current_file;
+	mainw->rec_avel=cfile->pb_fps/cfile->fps;
+	mainw->rec_aseek=(gdouble)mainw->pulsed->seek_pos/(gdouble)(cfile->arate*cfile->achans*cfile->asampsize/8);
+      }
     }
     return TRUE;
   }
@@ -2039,6 +2115,10 @@ static void *cache_my_audio(void *arg) {
 	  continue;
 	}
 
+#ifdef IS_MINGW
+	setmode(cbuffer->_fd, O_BINARY);
+#endif
+
 	g_free(filename);
     }
 
@@ -2076,7 +2156,7 @@ static void *cache_my_audio(void *arg) {
     
     if (cbuffer->_cbytesize<cbuffer->bytesize) {
       cbuffer->eof=TRUE;
-      cbuffer->_csamp_space=(long)((double)cbuffer->samp_space/(double)cbuffer->bytesize*(double)cbuffer->_cbytesize);
+      cbuffer->_csamp_space=(int64_t)((double)cbuffer->samp_space/(double)cbuffer->bytesize*(double)cbuffer->_cbytesize);
       cbuffer->samp_space=cbuffer->_csamp_space;
     }
 
@@ -2186,13 +2266,96 @@ lives_audio_buf_t *audio_cache_get_buffer(void) {
 }
 
 
+///////////////////////////////////////
+
+// plugin handling
+
+gboolean get_audio_from_plugin(float *fbuffer, int nchans, int arate, int nsamps) {
+  // get audio from an audio generator; fbuffer is filled with non-interleaved float
+
+  weed_timecode_t tc;
+
+  int error;
+
+  weed_plant_t *inst=rte_keymode_get_instance(mainw->agen_key,rte_key_getmode(mainw->agen_key));
+  weed_plant_t *filter=weed_get_plantptr_value(inst,"filter_class",&error);
+  weed_plant_t *channel=get_enabled_channel(inst,0,FALSE);
+
+  weed_process_f *process_func_ptr_ptr;
+  weed_process_f process_func;
+  
+
+  if (mainw->agen_needs_reinit) return FALSE; // wait for other thread to reinit us
+
+  // stop video thread from possibly interpolating/deiniting
+  if (pthread_mutex_trylock(&mainw->interp_mutex)) return FALSE;
+
+  // make sure values match, else we need to reinit the plugin
+  if (nchans!=weed_get_int_value(channel,"audio_channels",&error)||
+      arate!=weed_get_int_value(channel,"audio_rate",&error)||
+      weed_get_boolean_value(channel,"audio_interleaf",&error)==WEED_FALSE) {
+    // reinit plugin
+    mainw->agen_needs_reinit=TRUE;
+  }
+
+  weed_set_int_value(channel,"audio_channels",nchans);
+  weed_set_int_value(channel,"audio_rate",arate);
+  weed_set_boolean_value(channel,"audio_interleaf",WEED_TRUE);
+  weed_set_int_value(channel,"audio_data_length",nsamps);
+  weed_set_voidptr_value(channel,"audio_data",fbuffer);
+
+  weed_set_double_value(inst,"fps",cfile->pb_fps);
+
+  if (mainw->agen_needs_reinit) {
+    // allow main thread to complete the reinit so we do not delay; just return silence
+    pthread_mutex_unlock(&mainw->interp_mutex);
+    return FALSE;
+  }
+
+  tc=(double)mainw->agen_samps_count/(double)arate*U_SEC; // we take our timing from the number of samples read
+  weed_set_int64_value(channel,"timecode",tc);
+
+  weed_leaf_get(filter,"process_func",0,(void *)&process_func_ptr_ptr);
+  process_func=process_func_ptr_ptr[0];
+
+  if ((*process_func)(inst,tc)==WEED_ERROR_PLUGIN_INVALID) {
+    pthread_mutex_unlock(&mainw->interp_mutex);
+    return FALSE;
+  }
+  pthread_mutex_unlock(&mainw->interp_mutex);
+
+  mainw->agen_samps_count+=nsamps;
+
+  return TRUE;
+}
+
+
+void reinit_audio_gen(void) {
+  int agen_key=mainw->agen_key;
+  int ret;
+
+  weed_plant_t *inst=rte_keymode_get_instance(agen_key,rte_key_getmode(mainw->agen_key));
+
+  ret=weed_reinit_effect(inst,TRUE);
+  if (ret==FILTER_NO_ERROR||ret==FILTER_INFO_REINITED) {
+    mainw->agen_needs_reinit=FALSE;
+    mainw->agen_key=agen_key;
+  }
+}
+
 
 ////////////////////////////////////////
 // audio streaming
 
+#ifndef IS_MINGW
 static int astream_pid=0;
+#endif
 
 gboolean start_audio_stream(void) {
+#ifdef IS_MINGW
+  return FALSE;
+#else
+
   const gchar *playername="audiostreamer.pl";
   gchar *astream_name=NULL;
   gchar *astream_name_out=NULL;
@@ -2208,6 +2371,7 @@ gboolean start_audio_stream(void) {
   gboolean timeout=FALSE;
 
   astream_name=g_build_filename(prefs->tmpdir,astname,NULL);
+
   mkfifo(astream_name,S_IRUSR|S_IWUSR);
 
   astream_name_out=g_build_filename(prefs->tmpdir,astname_out,NULL);
@@ -2215,20 +2379,20 @@ gboolean start_audio_stream(void) {
   g_free(astname);
   g_free(astname_out);
 
-    if (prefs->audio_player==AUD_PLAYER_PULSE) {
+  if (prefs->audio_player==AUD_PLAYER_PULSE) {
 #ifdef HAVE_PULSE_AUDIO
-      arate=(int)mainw->pulsed->out_arate;
-      // TODO - chans, samps, signed, endian
+    arate=(int)mainw->pulsed->out_arate;
+    // TODO - chans, samps, signed, endian
 #endif
-    }
+  }
 
-    if (prefs->audio_player==AUD_PLAYER_JACK) {
+  if (prefs->audio_player==AUD_PLAYER_JACK) {
 #ifdef ENABLE_JACK
-      arate=(int)mainw->jackd->sample_out_rate;
-      // TODO - chans, samps, signed, endian
+    arate=(int)mainw->jackd->sample_out_rate;
+    // TODO - chans, samps, signed, endian
       
 #endif
-    }
+  }
 
 
   astream_pid=fork();
@@ -2260,6 +2424,10 @@ gboolean start_audio_stream(void) {
 
   lives_alarm_clear(alarm_handle);
 
+#ifdef IS_MINGW
+  setmode(afd, O_BINARY);
+#endif
+
   if (prefs->audio_player==AUD_PLAYER_PULSE) {
 #ifdef HAVE_PULSE_AUDIO
     mainw->pulsed->astream_fd=afd;
@@ -2272,15 +2440,21 @@ gboolean start_audio_stream(void) {
 #endif
   }
 
+#ifdef IS_MINGW
+  setmode(afd, O_BINARY);
+#endif
+
   g_free(astream_name);
   g_free(astream_name_out);
 
   return TRUE;
+#endif
 }
 
 
 
 void stop_audio_stream(void) {
+#ifndef IS_MINGW
   if (astream_pid>0) {
     // if we were streaming audio, kill it
     const gchar *playername="audiostreamer.pl";
@@ -2324,7 +2498,7 @@ void stop_audio_stream(void) {
 
   }
 
-
+#endif
 }
 
 
