@@ -1,14 +1,12 @@
 // callbacks.h
 // LiVES
-// (c) G. Finch <salsaman@xs4all.nl,salsaman@gmail.com> 2003 - 2011
+// (c) G. Finch <salsaman@xs4all.nl,salsaman@gmail.com> 2003 - 2012
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
 #ifndef HAS_LIVES_CALLBACKS_H
 #define HAS_LIVES_CALLBACKS_H
 
-#include <gtk/gtk.h>
-
 gboolean
 on_LiVES_delete_event                  (GtkWidget       *widget,
                                         GdkEvent        *event,
@@ -309,13 +307,9 @@ void
 on_spinbutton_end_value_changed          (GtkSpinButton   *spinbutton,
 					  gpointer         user_data);
 
-void
-on_ok_button4_clicked                     (GtkButton       *button,
-					   gpointer         user_data);
+void on_open_new_audio_clicked (GtkButton *, gpointer opt_filename);
 
-void
-on_load_audio_activate                  (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+void on_load_audio_activate (GtkMenuItem *, gpointer user_data);
 
 void on_load_subs_activate (GtkMenuItem *menuitem, gpointer user_data);
 
@@ -636,7 +630,10 @@ void
 on_showfct_activate               (GtkMenuItem     *menuitem,
 				   gpointer         user_data);
 
-void on_boolean_toggled(GtkObject *, gpointer user_data);
+void on_boolean_toggled(GObject *, gpointer user_data);
+
+
+void on_showsubs_toggled(GObject *, gpointer);
 
 void
 on_show_messages_activate            (GtkMenuItem     *menuitem,
