@@ -118,8 +118,8 @@ void *convert_swab_frame_thread(void *cc_params);
 boolean convert_layer_palette(weed_plant_t *layer, int outpl, int op_clamping);
 boolean convert_layer_palette_with_sampling(weed_plant_t *layer, int outpl, int out_sampling);
 boolean convert_layer_palette_full(weed_plant_t *layer, int outpl, int osamtype, boolean oclamping, int osubspace);
-boolean apply_gamma (weed_plant_t *ilayer, weed_plant_t *olayer, double gamma); ///< not used
-void resize_layer (weed_plant_t *layer, int width, int height, LiVESInterpType interp);
+//boolean apply_gamma (weed_plant_t *ilayer, weed_plant_t *olayer, double gamma); ///< not used
+boolean resize_layer (weed_plant_t *layer, int width, int height, LiVESInterpType interp);
 void letterbox_layer (weed_plant_t *layer, int width, int height, int nwidth, int nheight);
 void compact_rowstrides(weed_plant_t *layer);
 void create_empty_pixel_data(weed_plant_t *layer, boolean black_fill, boolean may_contig);
@@ -133,7 +133,7 @@ weed_plant_t *weed_layer_new(int width, int height, int *rowstrides, int current
 int weed_layer_get_palette(weed_plant_t *layer);
 
 cairo_t *layer_to_cairo(weed_plant_t *layer);
-gboolean cairo_to_layer(cairo_t *cairo, weed_plant_t *layer);
+boolean cairo_to_layer(cairo_t *cairo, weed_plant_t *layer);
 
 void alpha_unpremult(weed_plant_t *layer, boolean un);
 
@@ -154,6 +154,8 @@ boolean weed_palette_is_lower_quality(int p1, int p2);  ///< return TRUE if p1 i
 boolean weed_palette_is_resizable(int pal);
 gdouble weed_palette_get_compression_ratio (int pal);
 
+int get_weed_palette_for_cairo(void);
+
 #define BLACK_THRESH 20 ///< if R,G and B values are all <= this, we consider it a "black" pixel
 boolean lives_pixbuf_is_all_black(LiVESPixbuf *pixbuf);
 
