@@ -60,6 +60,11 @@ gboolean save_frame_index(gint fileno) {
       retval=do_write_failed_error_s_with_retry(fname,g_strerror(errno),NULL);
     }
     else {
+
+#ifdef IS_MINGW
+  setmode(fd, O_BINARY);
+#endif
+
       mainw->write_failed=FALSE;
       for (i=0;i<sfile->frames;i++) {
 	lives_write_le(fd,&sfile->frame_index[i],4,TRUE);
@@ -116,6 +121,10 @@ gboolean load_frame_index(gint fileno) {
       }
     }
     else {
+
+#ifdef IS_MINGW
+  setmode(fd, O_BINARY);
+#endif
       create_frame_index(fileno,FALSE,0,sfile->frames);
 
       mainw->read_failed=FALSE;
@@ -145,10 +154,16 @@ void del_frame_index(file *sfile) {
   // only done once all
 
   gchar *idxfile=g_build_filename(prefs->tmpdir,sfile->handle,"file_index",NULL);
-  gchar *com=g_strdup_printf("/bin/rm -f \"%s\"",idxfile);
+  gchar *com;
 
   register int i;
 
+#ifndef IS_MINGW
+  com=g_strdup_printf("/bin/rm -f \"%s\"",idxfile);
+#else
+  com=g_strdup_printf("rm.exe -f \"%s\"",idxfile);
+#endif
+
   // cannot call check_if_non_virtual() else we end up recursing
 
   if (sfile->frame_index!=NULL) {
@@ -304,7 +319,7 @@ boolean virtual_to_images(int sfileno, int sframe, int eframe, boolean update_pr
       } while (retval==LIVES_RETRY);
 
       if (oname!=NULL) g_free(oname);
-      if (pixbuf!=NULL) gdk_pixbuf_unref(pixbuf);
+      if (pixbuf!=NULL) lives_object_unref(pixbuf);
       pixbuf=NULL;
 
       if (retval==LIVES_CANCEL) return FALSE;
@@ -471,7 +486,11 @@ void clean_images_from_virtual (file *sfile, gint oldframes) {
       //      else {
 	// ...
       //}
+#ifndef IS_MINGW
       com=g_strdup_printf("/bin/rm -f \"%s\"",iname);
+#else
+      com=g_strdup_printf("rm.exe -f \"%s\"",iname);
+#endif
       lives_system(com,FALSE);
       g_free(com);
     }
