@@ -52,12 +52,24 @@ gchar *lives_fx_cat_to_text(lives_fx_cat_t cat, gboolean plural) {
   switch (cat) {
 
     // main categories
-  case LIVES_FX_CAT_GENERATOR:
+  case LIVES_FX_CAT_VIDEO_GENERATOR:
     if (!plural) return (g_strdup(_("generator")));
     else return (g_strdup(_("Generators")));
+  case LIVES_FX_CAT_AUDIO_GENERATOR:
+    if (!plural) return (g_strdup(_("audio generator")));
+    else return (g_strdup(_("Audio Generators")));
+  case LIVES_FX_CAT_AV_GENERATOR:
+    if (!plural) return (g_strdup(_("audio/video generator")));
+    else return (g_strdup(_("Audio/Video Generators")));
   case LIVES_FX_CAT_DATA_GENERATOR:
     if (!plural) return (g_strdup(_("data generator")));
     else return (g_strdup(_("Data Generators")));
+  case LIVES_FX_CAT_DATA_PROCESSOR:
+    if (!plural) return (g_strdup(_("data processor")));
+    else return (g_strdup(_("Data Processors")));
+  case LIVES_FX_CAT_DATA_SOURCE:
+    if (!plural) return (g_strdup(_("data source")));
+    else return (g_strdup(_("Data Sources")));
   case LIVES_FX_CAT_TRANSITION:
     if (!plural) return (g_strdup(_("transition")));
     else return (g_strdup(_("Transitions")));
@@ -178,7 +190,8 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
       mainw->show_procd=FALSE;
     }
     else cmd=g_strdup("fxrender");
-    fxcommand=g_strconcat ("smogrify ",cmd,"_",plugin_name," ", pdefault, (tmp=param_marshall (rfx, FALSE)), NULL);
+    fxcommand=g_strconcat (prefs->backend," \"",cmd,"_",plugin_name,"\" ", pdefault, 
+			   (tmp=param_marshall (rfx, FALSE)), NULL);
 
     g_free(plugin_name);
     g_free(cmd);
@@ -193,7 +206,6 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
       g_free(cfile->frame_index_back);
       cfile->frame_index_back=NULL;
     }
-
     lives_system(fxcommand,FALSE);
     g_free (fxcommand);
   }
@@ -290,9 +302,19 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
   }
   else cfile->fx_frame_pump=0;
 
+  if (rfx->props&RFX_PROPS_MAY_RESIZE||rfx->num_in_channels==0) {
+    if (rfx->status==RFX_STATUS_WEED) { 
+      // set out_channel dimensions for resizers / generators
+      int error;
+      weed_plant_t *first_out=get_enabled_channel((weed_plant_t *)rfx->source,0,FALSE);
+      weed_plant_t *first_ot=weed_get_plantptr_value(first_out,"template",&error);
+      weed_set_int_value(first_out,"width",weed_get_int_value(first_ot,"host_width",&error));
+      weed_set_int_value(first_out,"height",weed_get_int_value(first_ot,"host_height",&error));
+    }
+  }
+
   if (!do_progress_dialog(TRUE,TRUE,effectstring)||mainw->error) {
     mainw->last_dprint_file=ldfile;
-    do_rfx_cleanup(rfx);
     mainw->show_procd=TRUE;
     mainw->keep_pre=FALSE;
     if (mainw->error) {
@@ -308,12 +330,20 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     cfile->nokeep=FALSE;
     cfile->fx_frame_pump=0;
 
+    if (cfile->start==0) {
+      cfile->start=1;
+      cfile->end=cfile->frames;
+    }
+
     if (rfx->num_in_channels==0&&mainw->current_file!=current_file) {
       mainw->suppress_dprint=TRUE;
       close_current_file(current_file);
       mainw->suppress_dprint=FALSE;
     }
-    else mainw->current_file=current_file;
+    else {
+      mainw->current_file=current_file;
+      do_rfx_cleanup(rfx);
+    }
 
     mainw->is_generating=FALSE;
     mainw->no_switch_dprint=FALSE;
@@ -325,6 +355,11 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     return FALSE;
   }
 
+  if (cfile->start==0) {
+    cfile->start=1;
+    cfile->end=cfile->frames;
+  }
+
   do_rfx_cleanup(rfx);
 
   mainw->resizing=FALSE;
@@ -337,7 +372,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
     cfile->pb_fps=old_pb_fps;
     mainw->internal_messaging=FALSE;
     if (rfx->num_in_channels>0) gtk_widget_set_sensitive (mainw->select_last, TRUE);
-    if (rfx->num_in_channels>0) set_undoable (_ (rfx->menu_text),TRUE);
+    if (rfx->num_in_channels>0) set_undoable (rfx->menu_text,TRUE);
   }
 
   mainw->show_procd=TRUE;
@@ -392,7 +427,7 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 
   if (mainw->keep_pre) {
     // this comes from a preview which then turned into processing
-    gchar *com=g_strdup_printf("smogrify mv_pre \"%s\" %d %d \"%s\"",cfile->handle,cfile->progress_start,
+    gchar *com=g_strdup_printf("%s mv_pre \"%s\" %d %d \"%s\"",prefs->backend_sync, cfile->handle,cfile->progress_start,
 			       cfile->progress_end,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png");
 
     unlink(cfile->info_file);
@@ -536,7 +571,10 @@ gboolean do_effect(lives_rfx_t *rfx, gboolean is_preview) {
 
   if (!mainw->gen_to_clipboard) cfile->changed=TRUE;
   if (mainw->multitrack==NULL) {
-    if (new_file!=-1) switch_to_file ((mainw->current_file=0),new_file);
+    if (new_file!=-1) {
+      lives_sync();
+      switch_to_file ((mainw->current_file=0),new_file);
+    }
   }
   else {
     mainw->current_file=mainw->multitrack->render_file;
@@ -638,7 +676,7 @@ lives_render_error_t realfx_progress (gboolean reset) {
     }
   } while (retval==LIVES_RETRY);
   
-  gdk_pixbuf_unref (pixbuf);
+  lives_object_unref (pixbuf);
 
   if (cfile->clip_type==CLIP_TYPE_FILE) {
     cfile->frame_index[i-1]=-1;
@@ -647,7 +685,7 @@ lives_render_error_t realfx_progress (gboolean reset) {
   if (++i>cfile->end) {
     mainw->error=FALSE;
     mainw->cancelled=CANCEL_NONE;
-    com=g_strdup_printf ("smogrify mv_mgk \"%s\" %d %d \"%s\"",cfile->handle,cfile->start,
+    com=g_strdup_printf ("%s mv_mgk \"%s\" %d %d \"%s\"",prefs->backend,cfile->handle,cfile->start,
 			 cfile->end,cfile->img_type==IMG_TYPE_JPEG?"jpg":"png");
     lives_system (com,FALSE);
     g_free (com);
