@@ -16,8 +16,13 @@
 // general effects
 typedef enum {
   LIVES_FX_CAT_NONE=0,
-  LIVES_FX_CAT_GENERATOR,
+  LIVES_FX_CAT_VIDEO_GENERATOR,
+  LIVES_FX_CAT_AV_GENERATOR,
+  LIVES_FX_CAT_AUDIO_GENERATOR,
   LIVES_FX_CAT_DATA_GENERATOR,
+  LIVES_FX_CAT_DATA_VISUALISER,
+  LIVES_FX_CAT_DATA_PROCESSOR,
+  LIVES_FX_CAT_DATA_SOURCE,
   LIVES_FX_CAT_TRANSITION,
   LIVES_FX_CAT_AV_TRANSITION,
   LIVES_FX_CAT_VIDEO_TRANSITION,
