@@ -21,6 +21,9 @@ static size_t zero_buff_count=0;
 
 static gboolean seek_err;
 
+static size_t audio_read_inner(jack_driver_t *jackd, float **in_buffer, int fileno, 
+			       int nframes, double out_scale, gboolean rev_endian, gboolean out_unsigned, size_t rbytes);
+
 static gboolean check_zero_buff(size_t check_size) {
   if (check_size>zero_buff_count) {
     zero_buff=(unsigned char *)g_try_realloc(zero_buff,check_size);
@@ -241,6 +244,8 @@ static void jack_set_rec_avals(jack_driver_t *jackd, gboolean is_forward) {
 static void push_cache_buffer (lives_audio_buf_t *cache_buffer, jack_driver_t *jackd, 
 			       size_t in_bytes, size_t nframes, double shrink_factor) {
 
+  // push a cache_buffer for another thread to fill
+
   cache_buffer->fileno = jackd->playing_file;
   cache_buffer->seek = jackd->seek_pos;
   cache_buffer->bytesize = in_bytes;
@@ -267,6 +272,8 @@ static void push_cache_buffer (lives_audio_buf_t *cache_buffer, jack_driver_t *j
 
 
 static lives_audio_buf_t *pop_cache_buffer (void) {
+  // get next available cache_buffer
+
   lives_audio_buf_t *cache_buffer=audio_cache_get_buffer();
   if (cache_buffer!=NULL) return cache_buffer;
   return NULL;
@@ -281,15 +288,16 @@ static int audio_process (nframes_t nframes, void *arg) {
   jack_position_t pos;
   register int i;
   aserver_message_t *msg;
-  long xseek;
+  int64_t xseek;
   int new_file;
   gboolean from_memory=FALSE;
   gboolean wait_cache_buffer=FALSE;
+  gboolean pl_error=FALSE; ///< flag tells if we had an error during plugin processing
   lives_audio_buf_t *cache_buffer=NULL;
-  size_t nbytes;
+  size_t nbytes,rbytes;
 
 #ifdef DEBUG_AJACK
-  g_printerr("nframes %ld, sizeof(float) == %d\n", (long)nframes, sizeof(float));
+  g_printerr("nframes %ld, sizeof(float) == %d\n", (int64_t)nframes, sizeof(float));
 #endif
 
   if (!mainw->is_ready||jackd==NULL||(mainw->playing_file==-1&&jackd->is_silent&&jackd->msgq==NULL)) return 0;
@@ -300,7 +308,7 @@ static int audio_process (nframes_t nframes, void *arg) {
     // TODO - set seek_pos after setting record params
     switch (msg->command) {
     case ASERVER_CMD_FILE_OPEN:
-      new_file=atoi(msg->data);
+      new_file=atoi((char *)msg->data);
       if (jackd->playing_file!=new_file) {
 	jackd->playing_file=new_file;
       }
@@ -315,7 +323,7 @@ static int audio_process (nframes_t nframes, void *arg) {
       break;
     case ASERVER_CMD_FILE_SEEK:
       if (jackd->playing_file<0) break;
-      xseek=atol(msg->data);
+      xseek=atol((char *)msg->data);
       if (xseek<0.) xseek=0.;
       jackd->seek_pos=xseek;
       jackd->audio_ticks=mainw->currticks;
@@ -324,7 +332,7 @@ static int audio_process (nframes_t nframes, void *arg) {
     default:
       msg->data=NULL;
     }
-    if (msg->data!=NULL) g_free(msg->data);
+    if (msg->data!=NULL) g_free((char *)msg->data);
     msg->command=ASERVER_CMD_PROCESSED;
     if (msg->next==NULL) jackd->msgq=NULL; 
     else jackd->msgq = msg->next;
@@ -336,53 +344,62 @@ static int audio_process (nframes_t nframes, void *arg) {
   for (i = 0; i < jackd->num_output_channels; i++) 
     out_buffer[i] = (float *) jack_port_get_buffer(jackd->output_port[i], 
 						   nframes);
-
-  if (jackd->read_abuf==-1) {
-    // assign local copy from 
-    if (mainw->playing_file==-1 || ( cache_buffer = pop_cache_buffer() )==NULL || !cache_buffer->is_ready ) {
-      // audio buffer is not ready yet
-      if (!jackd->is_silent) {
-	for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
-	jackd->is_silent=TRUE;
+  if (mainw->agen_key==0||mainw->agen_needs_reinit||mainw->multitrack!=NULL) {
+    // if a plugin is generating audio we do not use cache_buffers, otherwise:
+    if (jackd->read_abuf==-1) {
+      // assign local copy from cache_buffers
+      if (mainw->playing_file==-1 || ( cache_buffer = pop_cache_buffer() )==NULL || !cache_buffer->is_ready ) {
+	// audio buffer is not ready yet
+	if (!jackd->is_silent) {
+	  for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
+	  jackd->is_silent=TRUE;
+	}
+	return 0;
       }
-      return 0;
+      if (cache_buffer->fileno==-1) jackd->playing_file=-1;
     }
-    if (cache_buffer->fileno==-1) jackd->playing_file=-1;
+    
+    if (cache_buffer!=NULL && cache_buffer->in_achans>0 && !cache_buffer->is_ready ) wait_cache_buffer=TRUE;
   }
 
-  if (cache_buffer!=NULL && cache_buffer->in_achans>0 && !cache_buffer->is_ready ) wait_cache_buffer=TRUE;
-
   jackd->state=jack_transport_query (jackd->client, &pos);
 
+
 #ifdef DEBUG_AJACK
   g_printerr("STATE is %d %d\n",jackd->state,jackd->play_when_stopped);
 #endif
 
   /* handle playing state */
   if (jackd->state==JackTransportRolling||jackd->play_when_stopped) {
-    gulong jackFramesAvailable = nframes; /* frames we have left to write to jack */
-    gulong inputFramesAvailable;          /* frames we have available this loop */
-    gulong numFramesToWrite;              /* num frames we are writing this loop */
-    glong in_frames=0;
-    gulong in_bytes=0;
+    uint64_t jackFramesAvailable = nframes; /* frames we have left to write to jack */
+    uint64_t inputFramesAvailable;          /* frames we have available this loop */
+    uint64_t numFramesToWrite;              /* num frames we are writing this loop */
+    int64_t in_frames=0;
+    uint64_t in_bytes=0,xin_bytes=0;
     gfloat shrink_factor=1.f;
     gdouble vol;
 
+    file *xfile=afile;
+
 #ifdef DEBUG_AJACK
     g_printerr("playing... jackFramesAvailable = %ld\n", jackFramesAvailable);
 #endif
 
     jackd->num_calls++;
 
-    if (!jackd->in_use||((jackd->playing_file<0||jackd->seek_pos<0.)&&jackd->read_abuf<0)||jackd->is_paused) {
+
+    if (!jackd->in_use||((jackd->playing_file<0||jackd->seek_pos<0.)&&jackd->read_abuf<0
+			  &&(mainw->agen_key==0||mainw->multitrack!=NULL))
+			 ||jackd->is_paused) {
+
       /* output silence if nothing is being outputted */
       if (!jackd->is_silent) for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
       jackd->is_silent=TRUE;
 
       // external streaming
-      nbytes=nframes*4;
-      check_zero_buff(nbytes);
-      audio_stream(zero_buff,nbytes,jackd->astream_fd);
+      rbytes=nframes*jackd->num_output_channels*2;
+      check_zero_buff(rbytes);
+      audio_stream(zero_buff,rbytes,jackd->astream_fd);
 
       if (!jackd->is_paused) jackd->frames_written+=nframes;
       if (jackd->seek_pos<0.&&jackd->playing_file>-1&&afile!=NULL) {
@@ -407,7 +424,13 @@ static int audio_process (nframes_t nframes, void *arg) {
 
       }
       else {
+
 	if (G_LIKELY(jackd->playing_file>=0)) {
+
+	  if (jackd->playing_file==mainw->ascrap_file&&mainw->playing_file>=-1&&mainw->files[mainw->playing_file]->achans>0) {
+	    xfile=mainw->files[mainw->playing_file];
+	  }
+
 	  in_bytes=ABS((in_frames=((gdouble)jackd->sample_in_rate/(gdouble)jackd->sample_out_rate*
 				   (gdouble)jackFramesAvailable+((gdouble)fastrand()/(gdouble)G_MAXUINT32))))
 	    *jackd->num_input_channels*jackd->bytes_per_channel;
@@ -455,14 +478,15 @@ static int audio_process (nframes_t nframes, void *arg) {
 	      }
 	    }
 
-	    if (!wait_cache_buffer) push_cache_buffer( cache_buffer, jackd, in_bytes, nframes, shrink_factor );
+	    if (!wait_cache_buffer&&((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL)) 
+	      push_cache_buffer( cache_buffer, jackd, in_bytes, nframes, shrink_factor );
 
 	    for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
 
 	    // external streaming
-	    nbytes=nframes*4;
-	    check_zero_buff(nbytes);
-	    audio_stream(zero_buff,nbytes,jackd->astream_fd);
+	    rbytes=nframes*jackd->num_output_channels*2;
+	    check_zero_buff(rbytes);
+	    audio_stream(zero_buff,rbytes,jackd->astream_fd);
 
 	    jackd->frames_written+=nframes;
 	    return 0;
@@ -471,7 +495,10 @@ static int audio_process (nframes_t nframes, void *arg) {
 	  else {
 	    if (shrink_factor>0.) {
 	      if (!(*jackd->cancelled)) {
-		gboolean eof=cache_buffer->eof;
+		gboolean eof=FALSE;
+
+		if (mainw->agen_key==0) eof=cache_buffer->eof;
+		else if (jackd->playing_file>-1 && xfile!=NULL && xfile->afilesize <=jackd->seek_pos) eof=TRUE;
 
 		if (eof) {
 		  // reached the end, forwards
@@ -501,7 +528,11 @@ static int audio_process (nframes_t nframes, void *arg) {
 	      }
 	    }
 	  }
-
+	  xin_bytes=in_bytes;
+	}
+	if (mainw->agen_key!=0&&mainw->multitrack==NULL) {
+	  in_bytes=jackFramesAvailable*jackd->num_output_channels*4;
+	  if (xin_bytes==0) xin_bytes=in_bytes;
 	}
 
 	if (!jackd->in_use||in_bytes==0) {
@@ -510,18 +541,21 @@ static int audio_process (nframes_t nframes, void *arg) {
 	  jackd->is_silent=TRUE;
 
 	  // external streaming
-	  nbytes=nframes*4;
-	  check_zero_buff(nbytes);
-	  audio_stream(zero_buff,nbytes,jackd->astream_fd);
+	  rbytes=nframes*jackd->num_output_channels*2;
+	  check_zero_buff(rbytes);
+	  audio_stream(zero_buff,rbytes,jackd->astream_fd);
 
 	  if (!jackd->is_paused) jackd->frames_written+=nframes;
-	  if (jackd->seek_pos<0.&&jackd->playing_file>-1&&afile!=NULL) {
-	    jackd->seek_pos+=nframes*afile->achans*afile->asampsize/8;
+	  if (jackd->seek_pos<0.&&jackd->playing_file>-1&&xfile!=NULL) {
+	    jackd->seek_pos+=nframes*xfile->achans*xfile->asampsize/8;
 	  }
 	  return 0;
 	}
 	
-	inputFramesAvailable = cache_buffer->samp_space;
+	if (mainw->multitrack!=NULL||(mainw->agen_key==0&&!mainw->agen_needs_reinit)) 
+	  inputFramesAvailable = cache_buffer->samp_space;
+	else inputFramesAvailable=jackFramesAvailable;
+
 #ifdef DEBUG_AJACK
 	g_printerr("%d inputFramesAvailable == %ld, %ld %ld,jackFramesAvailable == %ld\n", inputFramesAvailable, 
 		   in_frames,jackd->sample_in_rate,jackd->sample_out_rate,jackFramesAvailable);
@@ -533,7 +567,6 @@ static int audio_process (nframes_t nframes, void *arg) {
 #ifdef DEBUG_AJACK
 	g_printerr("nframes == %d, jackFramesAvailable == %ld,\n\tjackd->num_input_channels == %ld, jackd->num_output_channels == %ld, nf2w %ld, in_bytes %d, sf %.8f\n",  nframes, jackFramesAvailable, jackd->num_input_channels, jackd->num_output_channels, numFramesToWrite, in_bytes, shrink_factor);
 #endif
-	
 	jackd->frames_written+=numFramesToWrite;
 	jackFramesAvailable-=numFramesToWrite; /* take away what was written */
 	
@@ -541,130 +574,219 @@ static int audio_process (nframes_t nframes, void *arg) {
 	g_printerr("jackFramesAvailable == %ld\n", jackFramesAvailable);
 #endif
       }
+
       // playback from memory or file
       vol=mainw->volume*mainw->volume; // TODO - we should really use a logarithmic scale
       
-      if ( !from_memory && numFramesToWrite > 0 ) {
-	//	if (((gint)(jackd->num_calls/100.))*100==jackd->num_calls) if (mainw->soft_debug) g_print("audio pip\n");
-	if (cache_buffer->bufferf!=NULL) {
-	  for (i=0;i<jackd->num_output_channels;i++) {
-	    sample_move_d16_float(out_buffer[i], cache_buffer->buffer16[0] + i, numFramesToWrite, 
-				  jackd->num_output_channels, afile->signed_endian&AFORM_UNSIGNED, vol);
-	  }
+      if (numFramesToWrite) {
 
-	  if (jackd->astream_fd!=-1) {
-	    unsigned char *xbuf=(unsigned char *)cache_buffer->buffer16[0];
-	    // audio streaming if enabled
-	    nbytes=numFramesToWrite*4;
+	if (!from_memory) {
+	  //	if (((gint)(jackd->num_calls/100.))*100==jackd->num_calls) if (mainw->soft_debug) g_print("audio pip\n");
+	  if (mainw->agen_key!=0||mainw->agen_needs_reinit||cache_buffer->bufferf!=NULL) {
+	    float *fbuffer=NULL;
 
-	    if (jackd->num_output_channels!=2) {
-	      // need to remap channels to stereo (assumed for now)
-	      size_t bysize=4,tsize=0;
-	      unsigned char *inbuf=(unsigned char *)cache_buffer->buffer16[0];
-	      xbuf=(unsigned char *)g_try_malloc(nbytes);
-	      if (!xbuf) {
-		for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], numFramesToWrite);
+	    if (mainw->agen_key!=0||mainw->agen_needs_reinit) {
+	      // audio generated from plugin
+
+	      if (mainw->agen_needs_reinit) pl_error=TRUE;
+	      else {
+		fbuffer=(float *)g_malloc(numFramesToWrite*jackd->num_output_channels*4);
 		
-		// external streaming
-		nbytes=numFramesToWrite*4;
-		if (check_zero_buff(nbytes))
-		  audio_stream(zero_buff,nbytes,jackd->astream_fd);
-		return 0;
+		if (!get_audio_from_plugin(fbuffer,jackd->num_output_channels,jackd->sample_out_rate,numFramesToWrite)) {
+		  pl_error=TRUE;
+		}
 	      }
-	      if (jackd->num_output_channels==1) bysize=2;
-	      while (nbytes>0) {
-		memcpy(xbuf+tsize,inbuf,bysize);
-		tsize+=bysize;
-		nbytes-=bysize;
-		if (bysize==2) {
-		  // duplicate mono channel
+
+	      // get back interleaved float fbuffer; rate and channels should match
+
+	      for (i=0;i<jackd->num_output_channels;i++) {
+		if (pl_error) sample_silence_dS(out_buffer[i], nframes);
+		else {
+		  sample_move_float_float(out_buffer[i], fbuffer + i, numFramesToWrite,
+					  jackd->num_output_channels, vol);
+		}
+
+		
+	      }
+
+	      if (mainw->record&&mainw->ascrap_file!=-1&&mainw->playing_file>0) {
+		int out_unsigned=mainw->files[mainw->ascrap_file]->signed_endian&AFORM_UNSIGNED;
+		rbytes=numFramesToWrite*mainw->files[mainw->ascrap_file]->achans*
+		  mainw->files[mainw->ascrap_file]->asampsize>>3;
+
+		rbytes=audio_read_inner(jackd,out_buffer,mainw->ascrap_file,numFramesToWrite,1.0,
+					!(mainw->files[mainw->ascrap_file]->signed_endian&AFORM_BIG_ENDIAN),out_unsigned,rbytes);
+		mainw->files[mainw->ascrap_file]->aseek_pos+=rbytes;
+	      }
+
+	    }
+	    else {
+	      for (i=0;i<jackd->num_output_channels;i++) {
+		sample_move_d16_float(out_buffer[i], cache_buffer->buffer16[0] + i, numFramesToWrite, 
+				      jackd->num_output_channels, afile->signed_endian&AFORM_UNSIGNED, vol);
+	      }
+	    }
+	    
+	    if (jackd->astream_fd!=-1) {
+	      // audio streaming if enabled
+	      unsigned char *xbuf;
+
+	      nbytes=numFramesToWrite*jackd->num_output_channels*4;
+	      rbytes=numFramesToWrite*jackd->num_output_channels*2;
+
+	      if (pl_error) {
+		check_zero_buff(rbytes);
+		audio_stream(zero_buff,rbytes,jackd->astream_fd);
+	      }
+	      else {
+		if (mainw->agen_key==0)
+		  xbuf=(unsigned char *)cache_buffer->buffer16[0];
+		else {
+		  // plugin is generating and we are streaming: convert fbuffer to s16
+		  float **fp=(float **)g_malloc(jackd->num_output_channels*sizeof(float *));
+		  for (i=0;i<jackd->num_output_channels;i++) {
+		    fp[i]=fbuffer+i;
+		  }
+
+		  xbuf=(unsigned char *)g_malloc(nbytes*jackd->num_output_channels);
+
+		  sample_move_float_int((void *)xbuf,fp,numFramesToWrite,1.0,jackd->num_output_channels,16,0,TRUE,TRUE,1.0);
+
+		}
+	      
+		if (jackd->num_output_channels!=2) {
+		  // need to remap channels to stereo (assumed for now)
+		  size_t bysize=4,tsize=0;
+		  unsigned char *inbuf,*oinbuf=NULL;
+
+		  if (mainw->agen_key!=0) inbuf=(unsigned char *)cache_buffer->buffer16[0];
+		  else oinbuf=inbuf=xbuf;
+
+		  xbuf=(unsigned char *)g_try_malloc(nbytes);
+		  if (!xbuf) {
+		    // external streaming
+		    rbytes=numFramesToWrite*jackd->num_output_channels*2;
+		    if (check_zero_buff(rbytes))
+		      audio_stream(zero_buff,rbytes,jackd->astream_fd);
+		    return 0;
+		  }
+		  if (jackd->num_output_channels==1) bysize=2;
+		  while (nbytes>0) {
+		    memcpy(xbuf+tsize,inbuf,bysize);
+		    tsize+=bysize;
+		    nbytes-=bysize;
+		    if (bysize==2) {
+		      // duplicate mono channel
+		      memcpy(xbuf+tsize,inbuf,bysize);
+		      tsize+=bysize;
+		      nbytes-=bysize;
+		      inbuf+=bysize;
+		    }
+		    else {
+		      // or skip extra channels
+		      inbuf+=jackd->num_output_channels*4;
+		    }
+		  }
+		  nbytes=numFramesToWrite*jackd->num_output_channels*4;
+		  if (oinbuf!=NULL) g_free(oinbuf);
+		}
+		rbytes=numFramesToWrite*jackd->num_output_channels*2;
+		audio_stream(xbuf,rbytes,jackd->astream_fd);
+		if (mainw->agen_key!=0||xbuf!=(unsigned char *)cache_buffer->buffer16[0]) g_free(xbuf);
+	      }
+	    }
+	    
+	    if (fbuffer!=NULL) g_free(fbuffer);
+	    
+	  }
+	  else {
+	    for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], numFramesToWrite);
+	  
+	    // external streaming
+	    if (jackd->astream_fd!=-1) {
+	      rbytes=numFramesToWrite*jackd->num_output_channels*2;
+	      check_zero_buff(rbytes);
+	      audio_stream(zero_buff,rbytes,jackd->astream_fd);
+	    }
+	  }
+	}
+	else {
+	  if (jackd->read_abuf>-1&&!jackd->mute) {
+	    sample_move_abuf_float(out_buffer,jackd->num_output_channels,nframes,jackd->sample_out_rate,vol);
+	    
+	    if (jackd->astream_fd!=-1) {
+	      // audio streaming if enabled
+	      unsigned char *xbuf=(unsigned char *)out_buffer;
+	      nbytes=numFramesToWrite*jackd->num_output_channels*4;
+	      
+	      if (jackd->num_output_channels!=2) {
+		// need to remap channels to stereo (assumed for now)
+		size_t bysize=4,tsize=0;
+		unsigned char *inbuf=(unsigned char *)out_buffer;
+		xbuf=(unsigned char *)g_try_malloc(nbytes);
+		if (!xbuf) {
+		  for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], numFramesToWrite);
+		  
+		  // external streaming
+		  rbytes=numFramesToWrite*jackd->num_output_channels*2;
+		  if (check_zero_buff(rbytes))
+		    audio_stream(zero_buff,rbytes,jackd->astream_fd);
+		  return 0;
+		}
+		
+		if (jackd->num_output_channels==1) bysize=2;
+		while (nbytes>0) {
 		  memcpy(xbuf+tsize,inbuf,bysize);
 		  tsize+=bysize;
 		  nbytes-=bysize;
-		  inbuf+=bysize;
-		}
-		else {
-		  // or skip extra channels
-		  inbuf+=jackd->num_output_channels*4;
+		  if (bysize==2) {
+		    // duplicate mono channel
+		    memcpy(xbuf+tsize,inbuf,bysize);
+		    tsize+=bysize;
+		    nbytes-=bysize;
+		    inbuf+=bysize;
+		  }
+		  else {
+		    // or skip extra channels
+		    inbuf+=jackd->num_output_channels*4;
+		  }
 		}
+		nbytes=numFramesToWrite*jackd->num_output_channels*2;
 	      }
-	      nbytes=numFramesToWrite*4;
+	      rbytes=numFramesToWrite*jackd->num_output_channels*2;
+	      audio_stream(xbuf,rbytes,jackd->astream_fd);
+	      if (xbuf!=(unsigned char *)out_buffer) g_free(xbuf);
 	    }
-	    audio_stream(xbuf,nbytes,jackd->astream_fd);
-	    if (xbuf!=(unsigned char *)cache_buffer->buffer16[0]) g_free(xbuf);
 	  }
-
-	}
-	else {
-	  for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], numFramesToWrite);
-	  
-	  // external streaming
-	  nbytes=numFramesToWrite*4;
-	  check_zero_buff(nbytes);
-	  audio_stream(zero_buff,nbytes,jackd->astream_fd);
+	  else {
+	    for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
+	    
+	    // external streaming
+	    rbytes=numFramesToWrite*jackd->num_output_channels*2;
+	    check_zero_buff(rbytes);
+	    audio_stream(zero_buff,rbytes,jackd->astream_fd);
+	  }
 	}
       }
       else {
-	if (jackd->read_abuf>-1&&!jackd->mute) {
-	  sample_move_abuf_float(out_buffer,jackd->num_output_channels,nframes,jackd->sample_out_rate,vol);
-
-	  if (jackd->astream_fd!=-1) {
-	    // audio streaming if enabled
-	    unsigned char *xbuf=(unsigned char *)out_buffer;
-	    nbytes=numFramesToWrite*4;
-
-	    if (jackd->num_output_channels!=2) {
-	      // need to remap channels to stereo (assumed for now)
-	      size_t bysize=4,tsize=0;
-	      unsigned char *inbuf=(unsigned char *)out_buffer;
-	      xbuf=(unsigned char *)g_try_malloc(nbytes);
-	      if (!xbuf) {
-		for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], numFramesToWrite);
-		
-		// external streaming
-		nbytes=numFramesToWrite*4;
-		if (check_zero_buff(nbytes))
-		  audio_stream(zero_buff,nbytes,jackd->astream_fd);
-		return 0;
-	      }
-
-	      if (jackd->num_output_channels==1) bysize=2;
-	      while (nbytes>0) {
-		memcpy(xbuf+tsize,inbuf,bysize);
-		tsize+=bysize;
-		nbytes-=bysize;
-		if (bysize==2) {
-		  // duplicate mono channel
-		  memcpy(xbuf+tsize,inbuf,bysize);
-		  tsize+=bysize;
-		  nbytes-=bysize;
-		  inbuf+=bysize;
-		}
-		else {
-		  // or skip extra channels
-		  inbuf+=jackd->num_output_channels*4;
-		}
-	      }
-	      nbytes=numFramesToWrite*4;
-	    }
-	    audio_stream(xbuf,nbytes,jackd->astream_fd);
-	    if (xbuf!=(unsigned char *)out_buffer) g_free(xbuf);
-	  }
-	}
-	else {
-	  for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
+	// no input frames
+	nframes=jackFramesAvailable;
+	for(i = 0; i < jackd->num_output_channels; i++) sample_silence_dS(out_buffer[i], nframes);
 	  
-	  // external streaming
-	  nbytes=nframes*4;
-	  check_zero_buff(nbytes);
-	  audio_stream(zero_buff,nbytes,jackd->astream_fd);
-	}
+	// external streaming
+	rbytes=numFramesToWrite*jackd->num_output_channels*2;
+	check_zero_buff(rbytes);
+	audio_stream(zero_buff,rbytes,jackd->astream_fd);
       }
+
+
     }
 
+
     if (!from_memory) {
-      if (!wait_cache_buffer) push_cache_buffer( cache_buffer, jackd, in_bytes, nframes, shrink_factor );
-      if (shrink_factor>0.) jackd->seek_pos+=in_bytes;
+      if (!wait_cache_buffer&&mainw->agen_key==0) 
+	push_cache_buffer( cache_buffer, jackd, in_bytes, nframes, shrink_factor );
+      if (shrink_factor>0.) jackd->seek_pos+=xin_bytes;
     }
 
     /*    jackd->jack_pulse[0]=set_pulse(out_buffer[0],jack->buffer_size,8);
@@ -682,9 +804,10 @@ static int audio_process (nframes_t nframes, void *arg) {
 	sample_silence_dS(out_buffer[i] + (nframes - jackFramesAvailable), jackFramesAvailable);
 
       // external streaming
-      nbytes=jackFramesAvailable*4;
-      check_zero_buff(nbytes);
-      audio_stream(zero_buff,nbytes,jackd->astream_fd);
+      rbytes=numFramesToWrite*jackd->num_output_channels*2;
+
+      check_zero_buff(rbytes);
+      audio_stream(zero_buff,rbytes,jackd->astream_fd);
     }
 
   }
@@ -699,9 +822,9 @@ static int audio_process (nframes_t nframes, void *arg) {
     jackd->is_silent=TRUE;
 
     // external streaming
-    nbytes=nframes*4;
-    check_zero_buff(nbytes);
-    audio_stream(zero_buff,nbytes,jackd->astream_fd);
+    rbytes=nframes*jackd->num_output_channels*2;
+    check_zero_buff(rbytes);
+    audio_stream(zero_buff,rbytes,jackd->astream_fd);
 
     /* if we were told to reset then zero out some variables */
     /* and transition to STOPPED */
@@ -760,6 +883,39 @@ int lives_start_ready_callback (jack_transport_state_t state, jack_position_t *p
 }
 
 
+static size_t audio_read_inner(jack_driver_t *jackd, float **in_buffer, int ofileno, int nframes, 
+			       double out_scale, gboolean rev_endian, gboolean out_unsigned, size_t rbytes) {
+  int frames_out;
+
+  void *holding_buff=g_try_malloc(rbytes);
+
+  file *ofile=mainw->files[ofileno];
+
+  size_t bytes=0;
+
+  if (!holding_buff) return 0;
+
+  frames_out=sample_move_float_int((void *)holding_buff,in_buffer,nframes,out_scale,ofile->achans,
+				   ofile->asampsize,out_unsigned,rev_endian,FALSE,1.);
+
+  if (mainw->rec_samples>0) {
+    if (frames_out>mainw->rec_samples) frames_out=mainw->rec_samples;
+    mainw->rec_samples-=frames_out;
+  }
+
+  if (mainw->bad_aud_file==NULL) {
+    size_t target=frames_out*(ofile->asampsize/8)*ofile->achans;
+    // use write not lives_write - because of potential threading issues
+    bytes=write (mainw->aud_rec_fd,holding_buff,target);
+
+    if (bytes<target) mainw->bad_aud_file=filename_from_fd(NULL,mainw->aud_rec_fd);
+    if (bytes<0) bytes=0;
+  }
+
+  g_free(holding_buff);
+
+  return bytes;
+}
 
 
 static int audio_read (nframes_t nframes, void *arg) {
@@ -768,12 +924,11 @@ static int audio_read (nframes_t nframes, void *arg) {
   // this is the jack callback for when we are recording audio
 
   jack_driver_t* jackd = (jack_driver_t*)arg;
-  void *holding_buff;
   float *in_buffer[jackd->num_input_channels];
   float out_scale=(float)jackd->sample_in_rate/(float)afile->arate;
   int out_unsigned=afile->signed_endian&AFORM_UNSIGNED;
   int i;
-  long frames_out;
+  int64_t frames_out;
 
   size_t rbytes;
 
@@ -781,19 +936,15 @@ static int audio_read (nframes_t nframes, void *arg) {
 
   if (mainw->rec_samples==0) return 0; // wrote enough already, return until main thread stop
 
-  frames_out=(long)((gdouble)nframes/out_scale+1.);
-
+  frames_out=(int64_t)((gdouble)nframes/out_scale+1.);
   rbytes=frames_out*afile->achans*afile->asampsize/8;
-  holding_buff=g_try_malloc(rbytes);
-
-  if (!holding_buff) return 0;
 
   for (i=0;i<jackd->num_input_channels;i++) {
     in_buffer[i] = (float *) jack_port_get_buffer(jackd->input_port[i], nframes);
   }
 
-  frames_out=sample_move_float_int((void *)holding_buff,in_buffer,nframes,out_scale,afile->achans,
-				   afile->asampsize,out_unsigned,jackd->reverse_endian,1.);
+  rbytes=audio_read_inner(jackd,in_buffer,jackd->playing_file,nframes,out_scale,mainw->jackd_read->reverse_endian,
+			   out_unsigned,rbytes);
 
   jackd->frames_written+=nframes;
 
@@ -802,20 +953,6 @@ static int audio_read (nframes_t nframes, void *arg) {
     jackd->seek_pos+=rbytes;
   }
 
-  if (mainw->rec_samples>0) {
-    if (frames_out>mainw->rec_samples) frames_out=mainw->rec_samples;
-    mainw->rec_samples-=frames_out;
-  }
-
-  if (mainw->bad_aud_file==NULL) {
-    size_t target=frames_out*(afile->asampsize/8)*afile->achans,bytes;
-    // use write not lives_write - because of potential threading issues
-    bytes=write (mainw->aud_rec_fd,holding_buff,target);
-    if (bytes<target) mainw->bad_aud_file=filename_from_fd(NULL,mainw->aud_rec_fd);
-  }
-
-  g_free(holding_buff);
-
   if (mainw->rec_samples==0&&mainw->cancelled==CANCEL_NONE) mainw->cancelled=CANCEL_KEEP; // we wrote the required #
 
   return 0;
@@ -824,7 +961,7 @@ static int audio_read (nframes_t nframes, void *arg) {
 
 
 int jack_get_srate (nframes_t nframes, void *arg) {
-  //g_printerr("the sample rate is now %ld/sec\n", (long)nframes);
+  //g_printerr("the sample rate is now %ld/sec\n", (int64_t)nframes);
   return 0;
 }
 
@@ -847,7 +984,8 @@ void jack_shutdown(void* arg) {
   mainw->jackd=jack_get_driver(0,TRUE);
   mainw->jackd->msgq=NULL;
 
-  if (mainw->jackd->playing_file!=-1&&afile!=NULL) jack_audio_seek_bytes(mainw->jackd,mainw->jackd->seek_pos); // at least re-seek to the right place
+  if (mainw->jackd->playing_file!=-1&&afile!=NULL) 
+    jack_audio_seek_bytes(mainw->jackd,mainw->jackd->seek_pos); // at least re-seek to the right place
 }
 
 
@@ -912,6 +1050,8 @@ int jack_open_device(jack_driver_t *jackd) {
   jack_set_error_function(jack_error_func);
   jackd->client=NULL;
 
+  // TODO - use alarm
+
   stime=lives_get_current_ticks();
 
   while (jackd->client==NULL&&ntime<JACK_START_WAIT) {
@@ -1398,7 +1538,7 @@ gboolean jack_audio_seek_frame (jack_driver_t *jackd, gint frame) {
   // position will be adjusted to (floor) nearest sample
 
   volatile aserver_message_t *jmsg;
-  long seekstart;
+  int64_t seekstart;
   int alarm_handle=lives_alarm_set(LIVES_ACONNECT_TIMEOUT);
   gboolean timeout;
 
@@ -1415,20 +1555,20 @@ gboolean jack_audio_seek_frame (jack_driver_t *jackd, gint frame) {
   }
   lives_alarm_clear(alarm_handle);
   if (frame>afile->frames) frame=afile->frames;
-  seekstart=(long)((gdouble)(frame-1.)/afile->fps*afile->arate)*afile->achans*(afile->asampsize/8);
+  seekstart=(int64_t)((gdouble)(frame-1.)/afile->fps*afile->arate)*afile->achans*(afile->asampsize/8);
   jack_audio_seek_bytes(jackd,seekstart);
   return TRUE;
 }
 
 
-long jack_audio_seek_bytes (jack_driver_t *jackd, long bytes) {
+int64_t jack_audio_seek_bytes (jack_driver_t *jackd, int64_t bytes) {
   // seek to position "bytes" in current audio file
   // position will be adjusted to (floor) nearest sample
 
   // if the position is > size of file, we will seek to the end of the file
 
   volatile aserver_message_t *jmsg;
-  long seekstart;
+  int64_t seekstart;
 
   gboolean timeout;
   int alarm_handle=lives_alarm_set(LIVES_ACONNECT_TIMEOUT);
@@ -1451,7 +1591,7 @@ long jack_audio_seek_bytes (jack_driver_t *jackd, long bytes) {
   }
   lives_alarm_clear(alarm_handle);
 
-  seekstart=((long)(bytes/afile->achans/(afile->asampsize/8)))*afile->achans*(afile->asampsize/8);
+  seekstart=((int64_t)(bytes/afile->achans/(afile->asampsize/8)))*afile->achans*(afile->asampsize/8);
 
   if (seekstart<0) seekstart=0;
   if (seekstart>afile->afilesize) seekstart=afile->afilesize;
@@ -1522,7 +1662,7 @@ void jack_aud_pb_ready(gint fileno) {
       mainw->jackd->usigned=!asigned;
       mainw->jackd->seek_end=sfile->afilesize;
       
-      if ((aendian&&(G_BYTE_ORDER==G_BIG_ENDIAN))||(!aendian&&(G_BYTE_ORDER==G_LITTLE_ENDIAN))) 
+      if ((aendian&&(capable->byte_order==LIVES_BIG_ENDIAN))||(!aendian&&(capable->byte_order==LIVES_LITTLE_ENDIAN))) 
 	mainw->jackd->reverse_endian=TRUE;
       else mainw->jackd->reverse_endian=FALSE;
       
@@ -1554,6 +1694,7 @@ void jack_aud_pb_ready(gint fileno) {
 	mainw->rec_aseek=(gdouble)sfile->aseek_pos/(gdouble)(sfile->arate*sfile->achans*(sfile->asampsize/8));
       }
     }
+    if (mainw->agen_key!=0&&mainw->multitrack==NULL) mainw->jackd->in_use=TRUE; // audio generator is active
   }
 }
 
