@@ -427,9 +427,14 @@ void on_open_yuv4m_activate (GtkMenuItem *menuitem, gpointer user_data) {
     audio_fake=g_build_filename(prefs->tmpdir,cfile->handle,"audiodump.pcm",NULL);
 
 
+#ifndef IS_MINGW
     // fake file will go away when we close the current clip
     lives_system ((tmp=g_strdup_printf ("/bin/ln -s \"%s\" \"%s\" >/dev/null 2>&1",
 					audio_real,audio_fake)),TRUE);
+#else
+    // TODO
+#endif
+
     g_free(audio_real);
     g_free(audio_fake);
 
@@ -635,7 +640,8 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
   mkfifo(fifofile,S_IRUSR|S_IWUSR);
 
   if (!tvcardw->use_advanced) {
-    com=g_strdup_printf("smogrify open_tv_card \"%s\" \"%s\" \"%s\" \"%s\"",cfile->handle,chanstr,devstr,fifofile);
+    com=g_strdup_printf("%s open_tv_card \"%s\" \"%s\" \"%s\" \"%s\"",prefs->backend,cfile->handle,chanstr,
+			devstr,fifofile);
   }
   else {
     gdouble fps=0.;
@@ -652,8 +658,8 @@ on_live_tvcard_activate                      (GtkMenuItem     *menuitem,
     driver=g_strdup(gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(tvcardw->combod))->entry)));
     outfmt=g_strdup(gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(tvcardw->comboo))->entry)));
 
-    com=g_strdup_printf("smogrify open_tv_card \"%s\" \"%s\" \"%s\" \"%s\" %d %d %d %.3f \"%s\" \"%s\"",
-			cfile->handle,chanstr,
+    com=g_strdup_printf("%s open_tv_card \"%s\" \"%s\" \"%s\" \"%s\" %d %d %d %.3f \"%s\" \"%s\"",
+			prefs->backend,cfile->handle,chanstr,
 			devstr,fifofile,input,width,height,fps,driver,outfmt);
     g_free(driver);
     g_free(outfmt);
@@ -751,7 +757,7 @@ on_live_fw_activate                      (GtkMenuItem     *menuitem,
   unlink(fifofile);
   mkfifo(fifofile,S_IRUSR|S_IWUSR);
 
-  com=g_strdup_printf("smogrify open_fw_card \"%s\" %d %d \"%s\"",cfile->handle,cardno,cache,fifofile);
+  com=g_strdup_printf("%s open_fw_card \"%s\" %d %d \"%s\"",prefs->backend,cfile->handle,cardno,cache,fifofile);
   mainw->com_failed=FALSE;
   lives_system(com,FALSE);
   g_free(com);
