@@ -58,6 +58,34 @@ POSSIBILITY OF SUCH DAMAGES.
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#ifdef IS_MINGW
+
+#undef GDK_WINDOWING_X11
+#ifndef WINVER
+#define WINVER 0x0500
+#endif
+
+#include <windows.h>
+#include <winbase.h>
+#include <tlhelp32.h>
+#include <sddl.h>
+#include <sys/stat.h>
+
+#define O_SYNC (FILE_FLAG_NO_BUFFERING|FILE_FLAG_WRITE_THROUGH)
+
+typedef PROCESS_INFORMATION * lives_pid_t;
+typedef PROCESS_INFORMATION * lives_pgid_t;
+
+#include <gdk/gdkwin32.h>
+
+#else
+
+typedef pid_t lives_pid_t;
+typedef int lives_pgid_t;
+
+#endif
+
+
 #ifdef GDK_WINDOWING_X11
 #define USE_X11
 
@@ -77,13 +105,13 @@ POSSIBILITY OF SUCH DAMAGES.
 #include <assert.h>
 #include <errno.h>
 
+#define  __STDC_CONSTANT_MACROS
+#include <stdint.h>
+
 #include "widget-helper.h"
 
 #define QUOTEME(x) #x
 
-/// don't change this unless the backend is changed as well
-/// i.e. $GUI_BOOTSTRAP_FILE in smogrify
-#define BOOTSTRAP_NAME "/tmp/.smogrify"
 
 /// max files is actually 1 more than this, since file 0 is the clipboard
 #define MAX_FILES 65535
@@ -91,7 +119,12 @@ POSSIBILITY OF SUCH DAMAGES.
 /// this must match AC_PREFIX_DEFAULT in configure.in
 /// TODO - when lives-plugins is a separate package, use pkg-config to get PREFIX and remove PREFIX_DEFAULT
 #ifndef PREFIX_DEFAULT
+#ifndef IS_MINGW
 #define PREFIX_DEFAULT "/usr"
+#else
+// TODO - get this from the installer
+#define PREFIX_DEFAULT "C:\\Program Files\\LiVES"
+#endif
 #endif
 
 /// if --prefix= was not set, this is set to "NONE"
@@ -99,6 +132,9 @@ POSSIBILITY OF SUCH DAMAGES.
 #define PREFIX PREFIX_DEFAULT
 #endif
 
+
+
+#ifndef IS_MINGW
 #define DOC_DIR "/share/doc/lives-"
 
 #define THEME_DIR "/share/lives/themes/"
@@ -107,6 +143,19 @@ POSSIBILITY OF SUCH DAMAGES.
 #define ICON_DIR "/share/lives/icons/"
 #define DATA_DIR "/share/lives/"
 #define LIVES_CONFIG_DIR ".lives-dir/"
+#define LIVES_TMP_NAME "livestmp"
+
+#else // IS_MINGW
+#define DOC_DIR "\\Documents/"
+
+#define THEME_DIR "\\Themes/"
+#define PLUGIN_SCRIPTS_DIR "\\Plugins/"
+#define PLUGIN_EXEC_DIR "\\Plugins/"
+#define ICON_DIR "\\Icons/"
+#define DATA_DIR "\\Data/"
+#define LIVES_CONFIG_DIR "\\Config/"
+#define LIVES_TMP_NAME "livescache"
+#endif
 
 #define LIVES_DEVICE_DIR "/dev/"
 
@@ -159,8 +208,12 @@ POSSIBILITY OF SUCH DAMAGES.
 #include <limits.h>
 
 #ifndef PATH_MAX
+#ifdef MAX_PATH
+#define PATH_MAX MAX_PATH
+#else
 #define PATH_MAX 4096
 #endif
+#endif
 
 #ifdef __GNUC__
 #  define WARN_UNUSED  __attribute__((warn_unused_result))
@@ -379,7 +432,7 @@ typedef struct {
   guint signed_endian; ///< bitfield
 
   gint arate; ///< audio playback rate
-  gint64 unique_id;    ///< this and the handle can be used to uniquely id a file
+  guint64 unique_id;    ///< this and the handle can be used to uniquely id a file
   gint achans;
   gint asampsize;
 
@@ -406,8 +459,8 @@ typedef struct {
   gchar handle[256];
   gint ohsize;
   gint ovsize;
-  glong f_size;
-  glong afilesize;
+  int64_t f_size;
+  int64_t afilesize;
   gint old_frames; ///< for deordering, etc.
   gchar file_name[PATH_MAX]; ///< input file
   gchar info_file[PATH_MAX];
@@ -518,7 +571,7 @@ typedef struct {
 
   gboolean ratio_fps; ///< if the fps was set by a ratio
 
-  glong aseek_pos; ///< audio seek posn. (bytes) for when we switch clips
+  int64_t aseek_pos; ///< audio seek posn. (bytes) for when we switch clips
 
   // decoder data
 
@@ -563,11 +616,13 @@ typedef struct {
   // the following may need checking before use
   gboolean has_xmms;
   gboolean has_dvgrab;
-  gboolean has_sox;
+  gboolean has_sox_play;
+  gboolean has_sox_sox;
   gboolean has_autolives;
   gboolean has_mplayer;
   gboolean has_convert;
   gboolean has_composite;
+  gboolean has_identify;
   gboolean has_cdda2wav;
   gboolean has_midistartstop;
   gboolean has_jackd;
@@ -580,6 +635,11 @@ typedef struct {
   /// home directory - default location for config file - locale encoding
   gchar home_dir[PATH_MAX];
 
+  /// system tempdir (e.g /tmp for linux, C:\TEMP for win32)
+  gchar system_tmpdir[PATH_MAX];  ///< kept in locale encoding
+
+  gchar *rcfile;
+
   /// used for returning startup messages from the backend
   gchar startup_msg[256];
 
@@ -951,14 +1011,10 @@ void splash_end(void);
 void splash_msg(const gchar *msg, gdouble pct);
 void add_message_scroller(GtkWidget *conter);
 
-// utils.c
-#ifdef IS_IRIX
-void setenv(const char *name, const char *val, int _xx);
-#endif
 
 // system calls in utils.c
 int lives_system(const char *com, gboolean allow_error);
-pid_t lives_fork(const char *com);
+lives_pid_t lives_fork(const char *com);
 ssize_t lives_write(int fd, const void *buf, size_t count, gboolean allow_fail);
 ssize_t lives_write_le(int fd, const void *buf, size_t count, gboolean allow_fail);
 ssize_t lives_read(int fd, void *buf, size_t count, gboolean allow_less);
@@ -966,18 +1022,32 @@ ssize_t lives_read_le(int fd, void *buf, size_t count, gboolean allow_less);
 int lives_chdir(const char *path, gboolean allow_fail);
 int lives_fputs(const char *s, FILE *stream);
 char *lives_fgets(char *s, int size, FILE *stream);
+pid_t lives_getpid(void);
+int lives_getgid(void);
+int lives_getuid(void);
+void lives_freep(void **ptr);
+void lives_free(gpointer ptr);
+void lives_free_with_check(gpointer ptr);
+#ifdef IS_MINGW
+gboolean lives_win32_suspend_resume_process(DWORD pid, gboolean suspend);
+gboolean lives_win32_kill_subprocesses(DWORD pid, gboolean kill_parent);
+int lives_win32_get_num_logical_cpus(void);
+#endif
+int lives_kill(lives_pid_t pid, int sig);
+int lives_killpg(lives_pgid_t pgrp, int sig);
+void lives_srandom(unsigned int seed);
+uint64_t lives_random(void);
+ssize_t lives_readlink(const char *path, char *buf, size_t bufsiz);
+boolean lives_setenv(const char *name, const char *value);
+boolean lives_fsync(int fd);
+void lives_sync(void);
 
 char *filename_from_fd(char *val, int fd);
 
-
 float LEFloat_to_BEFloat(float f);
 uint64_t lives_10pow(int pow);
 int get_approx_ln(guint val);
-void lives_freep(void **ptr);
-void lives_free(gpointer ptr);
-void lives_free_with_check(gpointer ptr);
-int lives_kill(pid_t pid, int sig);
-int lives_killpg(int pgrp, int sig);
+
 int64_t lives_get_current_ticks(void);
 gboolean lives_alarm_get(int alarm_handle);
 int lives_alarm_set(int64_t ticks);
@@ -1037,9 +1107,9 @@ void prepare_to_play_foreign(void);
 gboolean after_foreign_play(void);
 gboolean check_file(const gchar *file_name, gboolean check_exists);  ///< check if file exists
 gboolean check_dir_access (const gchar *dir);
-gulong get_file_size(int fd);
-gulong sget_file_size(const gchar *name);
-gulong get_fs_free(const char *dir);
+uint64_t get_file_size(int fd);
+uint64_t sget_file_size(const gchar *name);
+uint64_t get_fs_free(const char *dir);
 gboolean is_writeable_dir(const char *dir);
 gboolean ensure_isdir(gchar *fname);
 gchar *ensure_extension(const gchar *fname, const gchar *ext) WARN_UNUSED;
@@ -1088,7 +1158,6 @@ void draw_little_bars (gdouble ptrtime);
 void set_sel_label (GtkWidget *label);
 void clear_mainw_msg (void);
 gint get_token_count (const gchar *string, int delim);
-GdkPixmap* gdk_pixmap_copy (GdkPixmap *pixmap);
 LiVESPixbuf *lives_pixbuf_new_blank(int width, int height, int palette);
 void get_border_size (GtkWidget *win, gint *bx, gint *by);
 gchar *g_strappend (gchar *string, gint len, const gchar *newbit);
@@ -1199,18 +1268,15 @@ void on_open_fw_activate (GtkMenuItem *menuitem, gpointer format);
 #define L2U8(String) ( g_locale_to_utf8 (String,-1,NULL,NULL,NULL) ) 
 
 
-#define PREFS_TIMEOUT 10000000 ///< 10 seconds // TODO !
+#define PREFS_TIMEOUT 10000000 ///< 10 seconds
 
 #define LIVES_TV_CHANNEL1 "http://www.serverwillprovide.com/sorteal/livestvclips/livestv.ogm"
 
 
 // round (double) a up to next (integer) multiple of (double) b
-// haha
 #define CEIL(a,b) ((int)(((double)a+(double)b-.000000001)/((double)b))*b)
 
 
-// should sprinkle some of these around
-
 gchar *dummychar;
 
 #define LIVES_NO_DEBUG
@@ -1248,7 +1314,7 @@ gchar *dummychar;
 
 #ifndef LIVES_FATAL
 #ifndef LIVES_NO_FATAL
-#define LIVES_FATAL(x)      fprintf(stderr, "LiVES fatal: %s\n", x)
+#define LIVES_FATAL(x)      {fprintf(stderr, "LiVES fatal: %s\n", x); raise (LIVES_SIGSEGV);}
 #else // LIVES_NO_FATAL
 #define LIVES_FATAL(x)      dummychar = x
 #endif // LIVES_NO_FATAL
