@@ -8,8 +8,11 @@
 #ifndef HAS_LIVES_MAINWINDOW_H
 #define HAS_LIVES_MAINWINDOW_H
 
+
 #include <pthread.h>
 
+#include "effects-data.h"
+
 #ifdef ALSA_MIDI
 #include <alsa/asoundlib.h>
 #endif
@@ -287,7 +290,7 @@ typedef struct {
 
   // recording from an external window
   guint foreign_key;
-  unsigned int foreign_id;
+  GdkNativeWindow foreign_id;
   GdkColormap *foreign_cmap;
   GdkPixmap *foreign_map;
   GdkWindow *foreign_window;
@@ -334,7 +337,7 @@ typedef struct {
   gint scr_width;
   gint scr_height;
   lives_toy_t toy_type;
-  pid_t toy_alives_pid; // 0, or thread for autolives toy
+  lives_pid_t toy_alives_pid; // 0, or thread for autolives toy
   gboolean autolives_reset_fx;
 
   gboolean toy_go_wild;
@@ -671,14 +674,10 @@ typedef struct {
   GtkWidget *framedraw_preview; ///< the 'redraw' button
   GtkWidget *framedraw_spinbutton; ///< the frame number button
   GtkWidget *framedraw_scale; ///< the slider
-  GtkWidget *framedraw_image; ///< the image
   GtkWidget *fd_frame; ///< surrounding frame widget
 
-  GdkPixmap *framedraw_orig_pixmap; ///< the original frame pixmap
-  GdkPixmap *framedraw_copy_pixmap; ///< the altered frame pixmap
-  GdkBitmap *framedraw_bitmap; ///< and its mask
-  GdkGC *framedraw_bitmapgc;  ///< the GC for the bitmap
-  GdkGC *framedraw_colourgc;  ///< the GC for the pixmap, we can draw in colours !
+  weed_plant_t *fd_layer_orig; ///< original layer uneffected
+  weed_plant_t *fd_layer; ///< framedraw preview layer
 
   // bars here -> actually text above bars
   GtkWidget *vidbar;
@@ -891,7 +890,7 @@ typedef struct {
 
   int aud_rec_fd; ///< fd of file we are recording audio to
   gdouble rec_end_time;
-  long rec_samples;
+  int64_t rec_samples;
   gdouble rec_fps;
   gint rec_vid_frames;
   gint rec_arate;
@@ -913,8 +912,6 @@ typedef struct {
   gboolean block_param_updates; ///< block visual param changes from updating real values
   gboolean no_interp; ///< block interpolation (for single frame previews)
 
-  gdouble fd_scale; ///< framedraw scale (image scaling downsize factor)
-
   weed_timecode_t cevent_tc; ///< timecode of currently processing event
 
   gboolean opening_multi; ///< flag to indicate multiple file selection
@@ -966,10 +963,6 @@ typedef struct {
   GIOChannel *iochan;
   GtkTextView *optextview;
 
-  /// error signalling during decoder plugin "rip_audio"
-  pthread_t sig_pid;
-  gchar *sig_file;
-
   gboolean has_custom_tools;
   gboolean has_custom_gens;
   gboolean has_custom_utilities;
@@ -1002,6 +995,11 @@ typedef struct {
   int64_t alarms[LIVES_MAX_ALARMS];
   int next_free_alarm;
 
+  // stuff specific to audio gens (will be extended to all rt audio fx)
+  volatile int agen_key; ///< which fx key is generating audio [1 based] (or 0 for none)
+  volatile gboolean agen_needs_reinit;
+  gint64 agen_samps_count; ///< count of samples since init
+
   gboolean aplayer_broken;
 
   gboolean com_failed;
@@ -1026,6 +1024,10 @@ typedef struct {
 
   float sepwin_scale;
 
+  lives_pconnect_t *pconx; ///< list of out -> in param connections
+  lives_cconnect_t *cconx; ///< list of out -> in alpha channel connections
+
+
 } mainwindow;
 
 GdkCursor *hidden_cursor;
@@ -1058,6 +1060,11 @@ extern GtkWidget *fx_dialog[2];
 
 
 #define LIVES_SIGKILL SIGKILL
+#define LIVES_SIGINT  SIGINT
+#define LIVES_SIGPIPE SIGPIPE
+#define LIVES_SIGUSR1 SIGUSR1
+#define LIVES_SIGABRT SIGABRT
+#define LIVES_SIGSEGV SIGSEGV
 #define LIVES_SIGHUP  SIGHUP
 #define LIVES_SIGTERM SIGTERM
 #define LIVES_SIGQUIT SIGQUIT
