@@ -4,11 +4,6 @@
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
-
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -281,6 +276,7 @@ void create_merge_dialog (void) {
   gtk_widget_show (hseparator);
   gtk_box_pack_start (GTK_BOX (vbox), hseparator, FALSE, TRUE, 0);
 
+  do_onchange_init(rfx);
 
   // now the dynamic part...
   merge_opts->param_vbox = gtk_vbox_new (FALSE, 0);
@@ -359,6 +355,9 @@ void on_trans_method_changed (GtkWidget *entry, gpointer user_data) {
 
   gtk_container_foreach (GTK_CONTAINER(merge_opts->param_vbox),bang,NULL);
   on_paramwindow_cancel_clicked (NULL,rfx);
+
+  do_onchange_init(rfx);
+
   make_param_box(GTK_VBOX (merge_opts->param_vbox), rfx);
   gtk_widget_show_all (merge_opts->param_vbox);
   merge_opts->align_start=!merge_opts->align_start;
@@ -525,13 +524,15 @@ on_merge_ok_clicked                   (GtkButton       *button,
 
 
   // number of frames to merge, must be <= selection length
-  if (!mainw->last_transition_loop_to_fit&&mainw->last_transition_align_start) { //loop_to_fit_audio
+  if (!mainw->last_transition_loop_to_fit) {
     cb_end=(clipboard->frames-excess_frames)*times_to_loop;
   }
   else {
+    //loop_to_fit_audio
     cb_end=cfile->end-cfile->start+1;
   } 
 
+
   // here we use undo_start and undo_end to mark the merged section,
   // insert_start and insert_end to mark the inserted section (if any)
   if (mainw->last_transition_align_start) {
@@ -550,10 +551,20 @@ on_merge_ok_clicked                   (GtkButton       *button,
     }
   }
 
-  cfile->progress_end=cb_end-cb_start+cfile->start+excess_frames*!mainw->last_transition_align_start;
-  cfile->progress_start=cfile->start;
-
-  clipboard->start=cb_start;
+  if (!mainw->last_transition_loop_to_fit) {
+    if (mainw->last_transition_align_start) {
+      cfile->progress_end=cb_end-cb_start+cfile->start+excess_frames*!mainw->last_transition_align_start;
+      cfile->progress_start=cfile->start;
+    }
+    else {
+      cfile->progress_end=cfile->end;
+      cfile->progress_start=cfile->end-cb_end+1;
+    }
+  }
+  else {
+    cfile->progress_start=cfile->start;
+    cfile->progress_end=cfile->end;
+  }
 
   // do the actual merge
   if (!do_effect (rfx,FALSE)) {
