@@ -639,7 +639,7 @@ struct _track_rect {
 
 /* translation table for matching event_id to init_event */
 typedef struct {
-  void *in;
+  uint64_t in;
   void *out;
 } ttable;
 
@@ -939,4 +939,18 @@ typedef enum {
   LMAP_INFO_SETNAME_CHANGED=1024
 } lives_lmap_error_t;
 
+
+
+// array max: TODO - use dynamic arrays to bypass track limits
+
+#ifdef IS_MINGW
+#define MAX_TRACKS 32768
+#define MAX_VIDEO_TRACKS 32768
+#define MAX_AUDIO_TRACKS 32768
+#else
+#define MAX_TRACKS 65536
+#define MAX_VIDEO_TRACKS 65536
+#define MAX_AUDIO_TRACKS 65536
+#endif
+
 #endif
