@@ -91,13 +91,17 @@ char **get_font_list(void) {
   return font_list;
 }
 
-int font_cmp(const void *p1, const void *p2) {  
+static int font_cmp(const void *p1, const void *p2) {  
   const char *s1 = (const char *)(*(char **)p1);
   const char *s2 = (const char *)(*(char **)p2);
-  return(strcasecmp(s1, s2));
+  char *u1 = g_utf8_casefold(s1, -1);
+  char *u2 = g_utf8_casefold(s2, -1);
+  int ret = strcmp(u1, u2);
+  g_free(u1);
+  g_free(u2);
+  return ret;
 }
 
-
 weed_plant_t *render_text_to_layer(weed_plant_t *layer, const char *text, const char *fontname,
   double size, lives_text_mode_t mode, lives_colRGBA32_t *fg_col, lives_colRGBA32_t *bg_col,
   gboolean center, gboolean rising, double top) {
@@ -217,7 +221,7 @@ static char *srt_read_text(FILE *pf, lives_subtitle_t *title) {
   char data[32768];
   size_t curlen;
 
-  long curpos;
+  int64_t curpos;
 
   if(!pf || !title)
     return(FALSE);
@@ -273,7 +277,7 @@ static char *sub_read_text(FILE *pf, lives_subtitle_t *title) {
   char data[32768];
   size_t curlen;
 
-  long curpos;
+  int64_t curpos;
 
   if(!pf || !title)
     return(FALSE);
@@ -713,7 +717,7 @@ static void parse_double_time(double tim, int *ph, int *pmin, int *psec, int *pm
 
 gboolean save_srt_subtitles(file *sfile, double start_time, double end_time, double offset_time, const char *filename) {
   lives_subtitles_t *subt=NULL;
-  long savepos = 0;
+  int64_t savepos = 0;
   FILE *pf;
   int num_saves;
   lives_subtitle_t *ptr = NULL;
@@ -780,7 +784,7 @@ gboolean save_srt_subtitles(file *sfile, double start_time, double end_time, dou
 
 gboolean save_sub_subtitles(file *sfile, double start_time, double end_time, double offset_time, const char *filename) {
   lives_subtitles_t *subt=NULL;
-  long savepos = 0;
+  int64_t savepos = 0;
   FILE *pf;
   int num_saves;
   lives_subtitle_t *ptr = NULL;
