@@ -36,6 +36,7 @@ GList *plugin_request_by_line (const gchar *plugin_type, const gchar *plugin_nam
 GList *plugin_request_by_space (const gchar *plugin_type, const gchar *plugin_name, const gchar *request);
 GList *plugin_request_common (const gchar *plugin_type, const gchar *plugin_name, const gchar *request, const gchar *delim, gboolean allow_blanks);
 
+typedef weed_plant_t *(*weed_bootstrap_f) (weed_default_getter_f *value, int num_versions, int *plugin_versions);
 
 /// video playback plugins
 typedef gboolean (*plugin_keyfunc) (gboolean down, guint16 unicode, guint16 keymod);
@@ -49,11 +50,13 @@ typedef struct {
   const char *(*module_check_init)(void);
   const char *(*version) (void);
   const char *(*get_description) (void);
-  const char *(*get_rfx) (void);
+
   gint *(*get_palette_list) (void);
   gboolean (*set_palette) (int palette);
   guint64 (*get_capabilities) (int palette);
-  gboolean (*render_frame) (int hsize, int vsize, int64_t timecode, void **pixel_data, void **return_data);
+
+  gboolean (*render_frame) (int hsize, int vsize, int64_t timecode, void **pixel_data, void **return_data,
+			    weed_plant_t **play_params);
 
   // optional
   gboolean (*init_screen) (int width, int height, gboolean fullscreen, uint64_t window_id, int argc, gchar **argv);
@@ -61,7 +64,12 @@ typedef struct {
   void (*module_unload) (void);
   const gchar *(*get_fps_list) (int palette);
   gboolean (*set_fps) (gdouble fps);
+  
+  const char *(*get_init_rfx) (void);
 
+  ///< optional (but should return a weed plantptr array of paramtmpl and chantmpl, NULL terminated)
+  const weed_plant_t **(*get_play_params) (weed_bootstrap_f f); 
+  
   // only for display plugins
   gboolean (*send_keycodes) (plugin_keyfunc);
 
@@ -100,6 +108,11 @@ typedef struct {
   int extra_argc;
   gchar **extra_argv;
 
+  const weed_plant_t **play_paramtmpls;
+  weed_plant_t **play_params;
+  weed_plant_t **alpha_chans;
+  int num_play_params;
+  int num_alpha_chans;
 
 } _vid_playback_plugin;
 
@@ -137,6 +150,7 @@ typedef struct {
 #define AUDIO_CODEC_RAW 7       // reserved
 #define AUDIO_CODEC_WMA2 8
 
+
 #define AUDIO_CODEC_MAX 31
   //
 #define AUDIO_CODEC_NONE 32
@@ -176,6 +190,8 @@ extern const char *anames[AUDIO_CODEC_MAX];
 #define LIVES_SEEK_QUALITY_LOSS (1<<2)
 
 
+// must be exactly the same as in decplugin.h
+
 typedef struct {
   gchar *URI; ///< the URI of this cdata
 
@@ -198,9 +214,9 @@ typedef struct {
   gint frame_width; // frame width and height are the size of the outer frame
   gint frame_height;
 
-  gfloat par; ///< pixel aspect ratio
+  float par; ///< pixel aspect ratio
 
-  gfloat fps;
+  float fps;
 
   int *palettes;
 
@@ -222,6 +238,11 @@ typedef struct {
 
   int seek_flag;
 
+#define SYNC_HINT_AUDIO_TRIM_START 1
+
+  int sync_hint;
+
+
   void *priv; ///< private data for demuxer/decoder - host should not touch this
 
 } lives_clip_data_t;
