@@ -287,6 +287,10 @@ typedef struct {
 
   gboolean force_system_clock; /// < force system clock (rather than soundcard) for timing ( better for high framerates )
 
+  gboolean force64bit;
+
+  gboolean auto_trim_audio;
+
   /** default 0; 1==use old (bad) behaviour on bigendian machines (r/w bigend ints/doubles); 2==bad reads, good writes */
   gint bigendbug;
 
@@ -315,6 +319,9 @@ typedef struct {
 #endif
 
   GList *disabled_decoders;
+
+  char backend_sync[PATH_MAX];
+  char backend[PATH_MAX];
 } _prefs;
 
 enum {
@@ -508,6 +515,7 @@ typedef struct {
   GtkWidget *checkbutton_ce_maxspect;
   GtkWidget *temp_label;
   GtkWidget *checkbutton_stream_audio;
+  GtkWidget *checkbutton_rec_after_pb;
   GtkTreeSelection *selection;
   gboolean needs_restart;
 } _prefsw;
@@ -578,6 +586,7 @@ void populate_combo_box(GtkComboBox *combo, GList *data);
 void set_combo_box_active_string(GtkComboBox *, gchar *active_str);
 
 void prefsw_set_astream_settings(_vid_playback_plugin *);
+void prefsw_set_rec_after_settings(_vid_playback_plugin *);
 
 void apply_button_set_enabled(GtkWidget *widget, gpointer func_data);
 
