@@ -26,7 +26,7 @@ static guint pulse_server_rate=0;
 
 #define PULSE_READ_BYTES 48000
 
-static guchar prbuf[PULSE_READ_BYTES];
+static guchar prbuf[PULSE_READ_BYTES*2];
 
 static size_t prb=0;
 
@@ -81,7 +81,7 @@ gboolean lives_pulse_init (short startup_phase) {
     LIVES_WARN("Unable to connect to pulseaudio server");
 
     if (!mainw->foreign) {
-      if (startup_phase==0&&capable->has_sox) {
+      if (startup_phase==0&&capable->has_sox_play) {
 	do_error_dialog_with_check(_("\nUnable to connect to pulse audio server.\nFalling back to sox audio player.\nYou can change this in Preferences/Playback.\n"),WARN_MASK_NO_PULSE_CONNECT);
 	switch_aud_to_sox(prefs->warning_mask&WARN_MASK_NO_PULSE_CONNECT);
       }
@@ -93,6 +93,7 @@ gboolean lives_pulse_init (short startup_phase) {
 	msg=g_strdup(_("\nUnable to connect to pulse audio server.\n"));
 	if (startup_phase!=2) {
 	  do_blocking_error_dialog(msg);
+	  mainw->aplayer_broken=TRUE;
 	}
 	else {
 	  msg2=g_strdup_printf("%s%s",msg,_("LiVES will exit and you can choose another audio player.\n"));
@@ -136,6 +137,7 @@ static void pulse_buff_free(void *ptr) {
 static void sample_silence_pulse (pulse_driver_t *pdriver, size_t nbytes, size_t xbytes) {
   guchar *buff;
   if (xbytes<=0) return;
+  if (mainw->aplayer_broken) return;
   while (nbytes>0) {
     if (nbytes<xbytes) xbytes=nbytes;
     buff=(guchar *)g_try_malloc0(xbytes);
@@ -162,11 +164,11 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
    static float old_volume=-1.;
    pulse_driver_t* pulsed = (pulse_driver_t*)arg;
 
-   gulong nframes=nbytes/pulsed->out_achans/(pulsed->out_asamps>>3);
+   uint64_t nframes=nbytes/pulsed->out_achans/(pulsed->out_asamps>>3);
 
    aserver_message_t *msg;
 
-   long seek,xseek;
+   int64_t seek,xseek;
    int new_file;
    gchar *filename;
    gboolean from_memory=FALSE;
@@ -180,6 +182,7 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 
    pa_volume_t pavol;
 
+
    pulsed->pstream=pstream;
 
    if (xbytes>nbytes) xbytes=nbytes;
@@ -190,11 +193,10 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
      return;
    }
 
-
    while ((msg=(aserver_message_t *)pulsed->msgq)!=NULL) {
      switch (msg->command) {
      case ASERVER_CMD_FILE_OPEN:
-       new_file=atoi(msg->data);
+       new_file=atoi((char *)msg->data);
        if (pulsed->playing_file!=new_file) {
 	 if (pulsed->is_opening) filename=g_build_filename(prefs->tmpdir,mainw->files[new_file]->handle,
 							   "audiodump.pcm",NULL);
@@ -207,6 +209,9 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	   pulsed->playing_file=-1;
 	 }
 	 else {
+#ifdef IS_MINGW
+	   setmode(pulsed->fd, O_BINARY);
+#endif
 	   pulsed->seek_pos=0;
 	   pulsed->playing_file=new_file;
 	   pulsed->audio_ticks=mainw->currticks;
@@ -227,9 +232,9 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
        break;
      case ASERVER_CMD_FILE_SEEK:
        if (pulsed->fd<0) break;
-       xseek=seek=atol(msg->data);
+       xseek=seek=atol((char *)msg->data);
        if (seek<0.) xseek=0.;
-       if (!pulsed->mute) {
+       if (mainw->agen_key==0&&!mainw->agen_needs_reinit) {
 	 lseek(pulsed->fd,xseek,SEEK_SET);
        }
        pulsed->seek_pos=seek;
@@ -240,7 +245,7 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
      default:
        msg->data=NULL;
      }
-     if (msg->data!=NULL) g_free(msg->data);
+     if (msg->data!=NULL) g_free((gpointer)msg->data);
      msg->command=ASERVER_CMD_PROCESSED;
      if (msg->next==NULL) pulsed->msgq=NULL;
      else pulsed->msgq = msg->next;
@@ -251,21 +256,26 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
    pulsed->state=pa_context_get_state(pulsed->con);
 
    if (pulsed->state==PA_CONTEXT_READY) {
-     gulong pulseFramesAvailable = nframes;
-     gulong inputFramesAvailable;
-     gulong numFramesToWrite;
-     glong in_frames=0;
-     gulong in_bytes=0;
+
+     uint64_t pulseFramesAvailable = nframes;
+     uint64_t inputFramesAvailable;
+     uint64_t numFramesToWrite;
+     int64_t in_frames=0;
+     uint64_t in_bytes=0,xin_bytes=0;
      gfloat shrink_factor=1.f;
      int swap_sign;
 
+     file *xfile=afile;
+
  #ifdef DEBUG_PULSE
      g_printerr("playing... pulseFramesAvailable = %ld\n", pulseFramesAvailable);
  #endif
 
      pulsed->num_calls++;
 
-     if (!pulsed->in_use||((pulsed->fd<0||pulsed->seek_pos<0.)&&pulsed->read_abuf<0)||pulsed->is_paused) {
+     if (!pulsed->in_use||(((pulsed->fd<0||pulsed->seek_pos<0.)&&pulsed->read_abuf<0)&&
+			   ((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL))
+			   ||pulsed->is_paused) {
 
        sample_silence_pulse(pulsed,nframes*pulsed->out_achans*(pulsed->out_asamps>>3),xbytes);
 
@@ -274,17 +284,20 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	 pulsed->seek_pos+=nframes*afile->achans*afile->asampsize/8;
 	 if (pulsed->seek_pos>=0) pulse_audio_seek_bytes(pulsed,pulsed->seek_pos);
        }
-       //g_print("pt a3\n");
+       //g_print("pt a3 %d\n",pulsed->in_use);
        return;
      }
 
-     if (G_LIKELY(pulseFramesAvailable>0&&(pulsed->read_abuf>-1||
-					   (pulsed->aPlayPtr!=NULL
-					    &&pulsed->in_achans>0)))) {
 
-       if (mainw->playing_file>-1&&pulsed->read_abuf>-1) {
+      if (G_LIKELY(pulseFramesAvailable>0&&(pulsed->read_abuf>-1||
+					   (pulsed->aPlayPtr!=NULL
+					    &&pulsed->in_achans>0)||
+					   ((mainw->agen_key!=0||mainw->agen_needs_reinit)&&mainw->multitrack==NULL) 
+					   ))) {
+
+        if (mainw->playing_file>-1&&pulsed->read_abuf>-1) {
 	 // playing back from memory buffers instead of from file
-	 // this is used in multitrack
+ 	 // this is used in multitrack
 	 from_memory=TRUE;
 
 	 numFramesToWrite=pulseFramesAvailable;
@@ -294,6 +307,12 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
        }
        else {
 	 if (G_LIKELY(pulsed->fd>=0)) {
+
+	   if ((pulsed->playing_file==mainw->ascrap_file&&!mainw->preview)&&mainw->playing_file>=-1
+	       &&mainw->files[mainw->playing_file]->achans>0) {
+	     xfile=mainw->files[mainw->playing_file];
+	   }
+
 	   pulsed->aPlayPtr->size=0;
 	   in_bytes=ABS((in_frames=((gdouble)pulsed->in_arate/(gdouble)pulsed->out_arate*
 				    (gdouble)pulseFramesAvailable+((gdouble)fastrand()/(gdouble)G_MAXUINT32))))
@@ -321,19 +340,21 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	       }
 	     }
 	     // rewind by in_bytes
-	     lseek(pulsed->fd,pulsed->seek_pos,SEEK_SET);
+	     if ((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL) 
+	       lseek(pulsed->fd,pulsed->seek_pos,SEEK_SET);
 	   }
 	   if G_UNLIKELY((in_bytes>pulsed->aPlayPtr->max_size && !(*pulsed->cancelled) && ABS(shrink_factor)<=100.f)) {
-	     pulsed->aPlayPtr->data=g_try_realloc(pulsed->aPlayPtr->data,in_bytes);
-	     if (pulsed->aPlayPtr->data!=NULL) {
-	       memset(pulsed->aPlayPtr->data,0,in_bytes);
-	       pulsed->aPlayPtr->max_size=in_bytes;
+	       pulsed->aPlayPtr->data=g_try_realloc(pulsed->aPlayPtr->data,in_bytes);
+	       if (pulsed->aPlayPtr->data!=NULL) {
+		 memset(pulsed->aPlayPtr->data,0,in_bytes);
+		 pulsed->aPlayPtr->max_size=in_bytes;
+	       }
 	     }
-	   }
 	   else {
 	     pulsed->aPlayPtr->max_size=0;
 	   }
-	   if (pulsed->mute||pulsed->aPlayPtr->data==NULL) {
+	   if (pulsed->mute||(pulsed->aPlayPtr->data==NULL&&((mainw->agen_key==0&&!mainw->agen_needs_reinit)||
+							     mainw->multitrack!=NULL))) {
 	     if (shrink_factor>0.f) pulsed->seek_pos+=in_bytes;
 	     if (pulsed->seek_pos>=pulsed->seek_end) {
 	       if (*pulsed->whentostop==STOP_ON_AUD_END) {
@@ -362,7 +383,10 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	       if (in_bytes>0) {
 		 // playing from a file
 		 if (!(*pulsed->cancelled)&&ABS(shrink_factor)<=100.f) {
-		   if ((pulsed->aPlayPtr->size=read(pulsed->fd,pulsed->aPlayPtr->data,in_bytes))==0) {
+		   if (((((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL)&&
+			 (pulsed->aPlayPtr->size=read(pulsed->fd,pulsed->aPlayPtr->data,in_bytes))==0))||
+		       (((mainw->agen_key!=0||mainw->agen_needs_reinit)&&mainw->multitrack==NULL&&
+			 pulsed->seek_pos+in_bytes>=xfile->afilesize))) {
 		     if (*pulsed->whentostop==STOP_ON_AUD_END) {
 		       *pulsed->cancelled=CANCEL_AUD_END;
 		     }
@@ -370,17 +394,21 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 		       loop_restart=TRUE;
 		       if (pulsed->loop==AUDIO_LOOP_PINGPONG) {
 			 pulsed->in_arate=-pulsed->in_arate;
-			 lseek(pulsed->fd,(pulsed->seek_pos-=in_bytes),SEEK_SET);
+			 if ((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL) 
+			   lseek(pulsed->fd,(pulsed->seek_pos-=in_bytes),SEEK_SET);
+			 else pulsed->seek_pos-=in_bytes;
 			 pulse_set_rec_avals(pulsed,FALSE);
 		       }
 		       else {
 			 if (pulsed->loop!=AUDIO_LOOP_NONE) {
 			   seek=0;
-			   lseek(pulsed->fd,(pulsed->seek_pos=seek),SEEK_SET);
+			   if ((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL) 
+			     lseek(pulsed->fd,(pulsed->seek_pos=seek),SEEK_SET);
+			   else pulsed->seek_pos=seek;
 			   pulse_set_rec_avals(pulsed,TRUE);
 			 }
 			 else {
-			   pulsed->in_use=FALSE;
+			   if (mainw->agen_key==0&&!mainw->agen_needs_reinit) pulsed->in_use=FALSE;
 			   loop_restart=FALSE;
 			   pulse_set_rec_avals(pulsed,TRUE);
 			 }
@@ -388,7 +416,8 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 		     }
 		   }
 		   else {
-		     if (pulsed->aPlayPtr->size<0) {
+		     if (pulsed->aPlayPtr->size<0&&((mainw->agen_key==0&&!mainw->agen_needs_reinit)||
+						    mainw->multitrack!=NULL)) {
 		       // read error...output silence
 		       sample_silence_pulse(pulsed,nframes*pulsed->out_achans*
 					    (pulsed->out_asamps>>3),xbytes);
@@ -397,16 +426,27 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 		     }
 		     if (shrink_factor<0.f) {
 		       // reverse play - rewind again by in_bytes
-		       lseek(pulsed->fd,pulsed->seek_pos,SEEK_SET);
+		       if ((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL) 
+			 lseek(pulsed->fd,pulsed->seek_pos,SEEK_SET);
+		     }
+		     else {
+		       if ((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL) 
+			 pulsed->seek_pos+=pulsed->aPlayPtr->size;
+		       else 
+			 pulsed->seek_pos+=in_bytes;
 		     }
-		     else pulsed->seek_pos+=pulsed->aPlayPtr->size;
 		   }
 		 }
 	       }
 	     } while (loop_restart);
 	   }
+	   xin_bytes=in_bytes;
 	 }
 
+	 if (mainw->agen_key!=0&&mainw->multitrack==NULL) {
+	   in_bytes=pulseFramesAvailable*pulsed->out_achans*2;
+	   if (xin_bytes==0) xin_bytes=in_bytes;
+	 }
 
 	 if (!pulsed->in_use||in_bytes==0) {
 	   // reached end of audio with no looping
@@ -421,43 +461,99 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	   return;
 	 }
 
-	 inputFramesAvailable = pulsed->aPlayPtr->size / (pulsed->in_achans * (pulsed->in_asamps>>3));
+	 if ((mainw->agen_key==0&&!mainw->agen_needs_reinit)||mainw->multitrack!=NULL) {
+	   swap_sign=afile->signed_endian&AFORM_UNSIGNED;
+
+	   inputFramesAvailable = pulsed->aPlayPtr->size / (pulsed->in_achans * (pulsed->in_asamps>>3));
  #ifdef DEBUG_PULSE
-	 g_printerr("%ld inputFramesAvailable == %ld, %ld, %ld %ld,pulseFramesAvailable == %ld\n", pulsed->aPlayPtr->size, inputFramesAvailable, in_frames, pulsed->in_arate,pulsed->out_arate,pulseFramesAvailable);
- #endif
-	 buffer = (guchar *)pulsed->aPlayPtr->data;
+	   g_printerr("%ld inputFramesAvailable == %ld, %ld, %ld %ld,pulseFramesAvailable == %ld\n", pulsed->aPlayPtr->size, inputFramesAvailable, in_frames, pulsed->in_arate,pulsed->out_arate,pulseFramesAvailable);
+#endif
+	   buffer = (guchar *)pulsed->aPlayPtr->data;
+	   
+	   numFramesToWrite = MIN(pulseFramesAvailable, (inputFramesAvailable/ABS(shrink_factor)+.001));
 
-	 numFramesToWrite = MIN(pulseFramesAvailable, (inputFramesAvailable/ABS(shrink_factor)+.001));
+#ifdef DEBUG_PULSE
+	   g_printerr("inputFramesAvailable after conversion %d\n",(uint64_t)((gdouble)inputFramesAvailable/shrink_factor+.001));
+	   g_printerr("nframes == %ld, pulseFramesAvailable == %ld,\n\tpulsed->num_input_channels == %ld, pulsed->out_achans == %ld\n",  nframes, pulseFramesAvailable, pulsed->in_achans, pulsed->out_achans);
+#endif
 
- #ifdef DEBUG_PULSE
-	 g_printerr("inputFramesAvailable after conversion %d\n",(gulong)((gdouble)inputFramesAvailable/shrink_factor+.001));
-	 g_printerr("nframes == %ld, pulseFramesAvailable == %ld,\n\tpulsed->num_input_channels == %ld, pulsed->out_achans == %ld\n",  nframes, pulseFramesAvailable, pulsed->in_achans, pulsed->out_achans);
- #endif
-
-	 swap_sign=afile->signed_endian&AFORM_UNSIGNED;
-
-	 if (pulsed->in_asamps==pulsed->out_asamps&&shrink_factor==1.&&pulsed->in_achans==pulsed->out_achans&&
-	     !pulsed->reverse_endian&&!swap_sign) {
-	   // no transformation needed
-	   pulsed->sound_buffer=buffer;
-	 }
-	 else {
-	   pulsed->sound_buffer=(guchar *)g_try_malloc0(pulsed->chunk_size);
-	   if (!pulsed->sound_buffer) {
-	     sample_silence_pulse(pulsed,nframes*pulsed->out_achans*
-				  (pulsed->out_asamps>>3),xbytes);
-	     if (!pulsed->is_paused) pulsed->frames_written+=nframes;
-	     return;
-	   }
-
-	   if (pulsed->in_asamps==8) {
-	     sample_move_d8_d16 ((short *)(pulsed->sound_buffer),(guchar *)buffer, numFramesToWrite, in_bytes, 
-				 shrink_factor, pulsed->out_achans, pulsed->in_achans, swap_sign?SWAP_U_TO_S:0);
+	   if (pulsed->in_asamps==pulsed->out_asamps&&shrink_factor==1.&&pulsed->in_achans==pulsed->out_achans&&
+	       !pulsed->reverse_endian&&!swap_sign) {
+	     // no transformation needed
+	     pulsed->sound_buffer=buffer;
 	   }
 	   else {
-	     sample_move_d16_d16((short*)pulsed->sound_buffer, (short*)buffer, numFramesToWrite, in_bytes, shrink_factor,
-				 pulsed->out_achans, pulsed->in_achans, pulsed->reverse_endian?SWAP_X_TO_L:0, 
-				 swap_sign?SWAP_U_TO_S:0);
+	     pulsed->sound_buffer=(guchar *)g_try_malloc0(pulsed->chunk_size);
+	     if (!pulsed->sound_buffer) {
+	       sample_silence_pulse(pulsed,nframes*pulsed->out_achans*
+				    (pulsed->out_asamps>>3),xbytes);
+	       if (!pulsed->is_paused) pulsed->frames_written+=nframes;
+	       return;
+	     }
+
+	     if (pulsed->in_asamps==8) {
+	       sample_move_d8_d16 ((short *)(pulsed->sound_buffer),(guchar *)buffer, numFramesToWrite, in_bytes, 
+				   shrink_factor, pulsed->out_achans, pulsed->in_achans, swap_sign?SWAP_U_TO_S:0);
+	     }
+	     else {
+	       sample_move_d16_d16((short*)pulsed->sound_buffer, (short*)buffer, numFramesToWrite, in_bytes, shrink_factor,
+				   pulsed->out_achans, pulsed->in_achans, pulsed->reverse_endian?SWAP_X_TO_L:0, 
+				   swap_sign?SWAP_U_TO_S:0);
+	     }
+	   }
+	 }
+	 else {
+	   // audio generator
+	   // get float audio from gen, convert it to S16
+	   float *fbuffer=NULL;
+	   gboolean pl_error=FALSE;
+	   xbytes=nbytes;
+	   numFramesToWrite=pulseFramesAvailable;
+	   
+
+	   if (mainw->agen_needs_reinit) pl_error=TRUE;
+	   else {
+	     fbuffer=(float *)g_malloc(numFramesToWrite*pulsed->out_achans*4);
+	     if (!get_audio_from_plugin(fbuffer,pulsed->out_achans,pulsed->out_arate,numFramesToWrite)) {
+	       pl_error=TRUE;
+	     }
+	   }
+	   
+	   if (!pl_error) {
+	     if (G_UNLIKELY(nbytes>pulsed->aPlayPtr->max_size)) {
+	       pulsed->aPlayPtr->data=g_try_realloc(pulsed->aPlayPtr->data,nbytes);
+	       if (pulsed->aPlayPtr->data!=NULL) {
+		 memset(pulsed->aPlayPtr->data,0,nbytes);
+		 pulsed->aPlayPtr->max_size=nbytes;
+	       }
+	       else pl_error=TRUE;
+	     }
+	     pulsed->aPlayPtr->size=nbytes;
+	   }
+
+	   // get back interleaved float fbuffer; rate and channels should match
+	   if (pl_error) nbytes=0;
+	   else {
+	     register int i;
+	     float **fp=(float**)lives_malloc(pulsed->out_achans*sizeof(float *));
+	     void *buf;
+	     pulsed->sound_buffer=(guchar *)pulsed->aPlayPtr->data;
+	     buf=(void *)pulsed->sound_buffer;
+
+	     for (i=0;i<pulsed->out_achans;i++) {
+	       fp[i]=fbuffer+i;
+	     }
+	     sample_move_float_int(buf,fp,numFramesToWrite,1.0,
+				   pulsed->out_achans,16,0,TRUE,TRUE,1.0);
+
+	     free(fp);
+
+	     if (mainw->record&&mainw->ascrap_file!=-1&&mainw->playing_file>0) {
+		size_t rbytes=numFramesToWrite*mainw->files[mainw->ascrap_file]->achans*
+		  mainw->files[mainw->ascrap_file]->asampsize>>3;
+		pulse_flush_read_data(pulsed,mainw->ascrap_file,nbytes,mainw->files[mainw->ascrap_file]->signed_endian&AFORM_BIG_ENDIAN,buf);
+		mainw->files[mainw->ascrap_file]->aseek_pos+=rbytes;
+	     }
 	   }
 	 }
 
@@ -496,12 +592,13 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	       if (!pulsed->is_paused) pulsed->frames_written+=nframes;
 	       return;
 	     }
-	     lives_memcpy(buffer,pulsed->sound_buffer+offs,xbytes);
+	     if (pulsed->sound_buffer) lives_memcpy(buffer,pulsed->sound_buffer+offs,xbytes);
 	     offs+=xbytes;
 	     needs_free=TRUE;
 	   }
 	   if (pulsed->astream_fd!=-1) audio_stream(buffer,xbytes,pulsed->astream_fd);
-	   pa_stream_write(pulsed->pstream,buffer,xbytes,buffer==pulsed->aPlayPtr->data?NULL:pulse_buff_free,0,PA_SEEK_RELATIVE);
+	   pa_stream_write(pulsed->pstream,buffer,xbytes,buffer==pulsed->aPlayPtr->data?NULL:
+			   pulse_buff_free,0,PA_SEEK_RELATIVE);
 	 }
 	 else {
 	   if (pulsed->read_abuf>-1&&!pulsed->mute) {
@@ -518,8 +615,10 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 	nbytes-=xbytes;
       }
       
-      if (needs_free&&pulsed->sound_buffer!=pulsed->aPlayPtr->data) g_free(pulsed->sound_buffer);
-      
+       if (needs_free&&pulsed->sound_buffer!=pulsed->aPlayPtr->data&&pulsed->sound_buffer!=NULL) {
+	 g_free(pulsed->sound_buffer);
+	 pulsed->sound_buffer=NULL;
+       }
     }
 
     if(pulseFramesAvailable) {
@@ -549,41 +648,64 @@ static void pulse_audio_write_process (pa_stream *pstream, size_t nbytes, void *
 
 
 
-void pulse_flush_read_data(pulse_driver_t *pulsed, size_t rbytes, void *data) {
+size_t pulse_flush_read_data(pulse_driver_t *pulsed, int fileno, size_t rbytes, gboolean rev_endian, void *data) {
   short *gbuf;
-  size_t bytes_out,frames_out;
+  size_t bytes_out,frames_out,bytes=0;
   void *holding_buff;
-  float out_scale=(float)pulsed->in_arate/(float)afile->arate;
 
-  int swap_sign=afile->signed_endian&AFORM_UNSIGNED;
+  float out_scale;
+  int swap_sign;
 
-  if (prb==0||mainw->rec_samples==0) return;
+  file *ofile;
 
-  gbuf=(short *)g_try_malloc(prb);
 
-  if (!gbuf) return;
+  if (mainw->agen_key==0&&!mainw->agen_needs_reinit) {
+    if (prb==0||mainw->rec_samples==0) return 0;
+    if (prb<=PULSE_READ_BYTES*2&&rbytes>0) {
+      gbuf=(short *)data;
+    }
+    else {
+      gbuf=(short *)g_try_malloc(prb);
+      if (!gbuf) return 0;
+      lives_memcpy(gbuf,prbuf,prb-rbytes);
+      if (rbytes>0) lives_memcpy(gbuf+(prb-rbytes)/sizeof(short),data,rbytes);
+    }
+    ofile=afile;
+  }
+  else {
+    if (rbytes==0) return 0;
+    if (fileno==-1) return 0;
+    gbuf=(short *)data;
+    prb=rbytes;
+    ofile=mainw->files[fileno];
+  }
 
-  lives_memcpy(gbuf,prbuf,prb-rbytes);
-  if (rbytes>0) lives_memcpy(gbuf+(prb-rbytes)/sizeof(short),data,rbytes);
+  out_scale=(float)pulsed->in_arate/(float)ofile->arate;
+  swap_sign=ofile->signed_endian&AFORM_UNSIGNED;
 
-  frames_out=(size_t)((gdouble)((prb/(pulsed->in_asamps>>3)/pulsed->in_achans))/out_scale+.5);
+  frames_out=(size_t)((gdouble)((prb/(ofile->asampsize>>3)/ofile->achans))/out_scale+.5);
 
-  bytes_out=frames_out*afile->achans*(afile->asampsize>>3);
+  if (mainw->agen_key==0&&!mainw->agen_needs_reinit) {
+    if (frames_out != pulsed->chunk_size) pulsed->chunk_size = frames_out;
+  }
+
+  bytes_out=frames_out*ofile->achans*(ofile->asampsize>>3);
 
   holding_buff=g_try_malloc(bytes_out);
 
-  if (!holding_buff) return;
+  if (!holding_buff) return 0;
 
-  if (frames_out != pulsed->chunk_size) pulsed->chunk_size = frames_out;
-
-  if (afile->asampsize==16) {
-    sample_move_d16_d16((short *)holding_buff,gbuf,frames_out,prb,out_scale,afile->achans,pulsed->in_achans,pulsed->reverse_endian?SWAP_L_TO_X:0,swap_sign?SWAP_S_TO_U:0);
+  if (ofile->asampsize==16) {
+    sample_move_d16_d16((short *)holding_buff,gbuf,frames_out,prb,out_scale,ofile->achans,pulsed->in_achans,pulsed->reverse_endian?SWAP_L_TO_X:0,swap_sign?SWAP_S_TO_U:0);
   }
   else {
-    sample_move_d16_d8((uint8_t *)holding_buff,gbuf,frames_out,prb,out_scale,afile->achans,pulsed->in_achans,swap_sign?SWAP_S_TO_U:0);
+    sample_move_d16_d8((uint8_t *)holding_buff,gbuf,frames_out,prb,out_scale,ofile->achans,pulsed->in_achans,swap_sign?SWAP_S_TO_U:0);
   }
     
-  g_free(gbuf);
+  if (gbuf!=(short *)data) {
+    g_free(gbuf);
+  }
+
   prb=0;
 
   if (mainw->rec_samples>0) {
@@ -592,7 +714,7 @@ void pulse_flush_read_data(pulse_driver_t *pulsed, size_t rbytes, void *data) {
   }
 
   if (mainw->bad_aud_file==NULL) {
-    size_t target=frames_out*(afile->asampsize/8)*afile->achans,bytes;
+    size_t target=frames_out*(ofile->asampsize/8)*ofile->achans,bytes;
     // use write not lives_write - because of potential threading issues
     bytes=write (mainw->aud_rec_fd,holding_buff,target);
     if (bytes<target) mainw->bad_aud_file=filename_from_fd(NULL,mainw->aud_rec_fd);
@@ -600,6 +722,7 @@ void pulse_flush_read_data(pulse_driver_t *pulsed, size_t rbytes, void *data) {
 
   g_free(holding_buff);
 
+  return bytes;
 }
 
 
@@ -640,7 +763,11 @@ static void pulse_audio_read_process (pa_stream *pstream, size_t nbytes, void *a
     return;
   }
 
-  pulse_flush_read_data(pulsed,rbytes,data);
+  if (prb<=PULSE_READ_BYTES*2) {
+    lives_memcpy(&prbuf[prb-rbytes],data,rbytes);
+    pulse_flush_read_data(pulsed,mainw->playing_file,prb,pulsed->reverse_endian,prbuf);
+  }
+  else pulse_flush_read_data(pulsed,mainw->playing_file,rbytes,pulsed->reverse_endian,data);
 
   pa_stream_drop(pulsed->pstream);
 
@@ -712,6 +839,7 @@ int pulse_audio_init(void) {
   pulsed.pstream=NULL;
   pulsed.pa_props=NULL;
   pulsed.playing_file=-1;
+  pulsed.sound_buffer=NULL;
   return 0;
 }
 
@@ -741,6 +869,7 @@ int pulse_audio_read_init(void) {
   pulsed_reader.is_paused=FALSE;
   pulsed_reader.pstream=NULL;
   pulsed_reader.pa_props=NULL;
+  pulsed_reader.sound_buffer=NULL;
 
   return 0;
 }
@@ -786,7 +915,7 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
   pdriver->in_asamps=pdriver->out_asamps=16;
   pdriver->out_signed=0;
 
-  if (G_BYTE_ORDER==G_BIG_ENDIAN) pdriver->out_endian=AFORM_BIG_ENDIAN;
+  if (capable->byte_order==LIVES_BIG_ENDIAN) pdriver->out_endian=AFORM_BIG_ENDIAN;
   else pdriver->out_endian=AFORM_LITTLE_ENDIAN;
 
   if (pdriver->is_output) {
@@ -829,9 +958,14 @@ int pulse_driver_activate(pulse_driver_t *pdriver) {
 
 #ifdef PA_STREAM_START_UNMUTED
     pa_stream_connect_playback(pdriver->pstream,NULL,&pa_battr,
-			       (pa_stream_flags_t)(PA_STREAM_START_UNMUTED|PA_STREAM_ADJUST_LATENCY|PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE),&out_vol,NULL);
+			       (pa_stream_flags_t)(PA_STREAM_START_UNMUTED|PA_STREAM_ADJUST_LATENCY|
+						   PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE),
+			       &out_vol,NULL);
 #else
-    pa_stream_connect_playback(pdriver->pstream,NULL,&pa_battr,(pa_stream_flags_t)(PA_STREAM_ADJUST_LATENCY|PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE),&out_vol,NULL);
+    pa_stream_connect_playback(pdriver->pstream,NULL,&pa_battr,(pa_stream_flags_t)(PA_STREAM_ADJUST_LATENCY|
+										   PA_STREAM_INTERPOLATE_TIMING|
+										   PA_STREAM_AUTO_TIMING_UPDATE),
+			       &out_vol,NULL);
 #endif
 
     while (pa_stream_get_state(pdriver->pstream)!=PA_STREAM_READY) {
@@ -929,7 +1063,7 @@ gdouble lives_pulse_get_pos(pulse_driver_t *pulsed) {
 gboolean pulse_audio_seek_frame (pulse_driver_t *pulsed, gint frame) {
   // seek to frame "frame" in current audio file
   // position will be adjusted to (floor) nearest sample
-  long seekstart;
+  int64_t seekstart;
   volatile aserver_message_t *pmsg;
   int alarm_handle=lives_alarm_set(LIVES_ACONNECT_TIMEOUT);
   gboolean timeout;
@@ -951,13 +1085,13 @@ gboolean pulse_audio_seek_frame (pulse_driver_t *pulsed, gint frame) {
   }
   lives_alarm_clear(alarm_handle);
   if (frame>afile->frames) frame=afile->frames;
-  seekstart=(long)((gdouble)(frame-1.)/afile->fps*afile->arate)*afile->achans*(afile->asampsize/8);
+  seekstart=(int64_t)((gdouble)(frame-1.)/afile->fps*afile->arate)*afile->achans*(afile->asampsize/8);
   pulse_audio_seek_bytes(pulsed,seekstart);
   return TRUE;
 }
 
 
-long pulse_audio_seek_bytes (pulse_driver_t *pulsed, long bytes) {
+int64_t pulse_audio_seek_bytes (pulse_driver_t *pulsed, int64_t bytes) {
   // seek to position "bytes" in current audio file
   // position will be adjusted to (floor) nearest sample
 
@@ -967,7 +1101,7 @@ long pulse_audio_seek_bytes (pulse_driver_t *pulsed, long bytes) {
   gboolean timeout;
   int alarm_handle=lives_alarm_set(LIVES_ACONNECT_TIMEOUT);
 
-  long seekstart;
+  int64_t seekstart;
 
   if (alarm_handle==-1) {
     LIVES_WARN("Invalid alarm handle");
@@ -985,7 +1119,7 @@ long pulse_audio_seek_bytes (pulse_driver_t *pulsed, long bytes) {
   }
   lives_alarm_clear(alarm_handle);
 
-  seekstart=((long)(bytes/afile->achans/(afile->asampsize/8)))*afile->achans*(afile->asampsize/8);
+  seekstart=((int64_t)(bytes/afile->achans/(afile->asampsize/8)))*afile->achans*(afile->asampsize/8);
 
   if (seekstart<0) seekstart=0;
   if (seekstart>afile->afilesize) seekstart=afile->afilesize;
@@ -1057,7 +1191,7 @@ void pulse_aud_pb_ready(gint fileno) {
       if (sfile->opening) mainw->pulsed->is_opening=TRUE;
       else mainw->pulsed->is_opening=FALSE;
       
-      if ((aendian&&(G_BYTE_ORDER==G_BIG_ENDIAN))||(!aendian&&(G_BYTE_ORDER==G_LITTLE_ENDIAN))) 
+      if ((aendian&&(capable->byte_order==LIVES_BIG_ENDIAN))||(!aendian&&(capable->byte_order==LIVES_LITTLE_ENDIAN))) 
 	mainw->pulsed->reverse_endian=TRUE;
       else mainw->pulsed->reverse_endian=FALSE;
       
@@ -1088,6 +1222,7 @@ void pulse_aud_pb_ready(gint fileno) {
 	mainw->rec_aseek=(gdouble)sfile->aseek_pos/(gdouble)(sfile->arate*sfile->achans*(sfile->asampsize/8));
       }
     }
+    if (mainw->agen_key!=0&&mainw->multitrack==NULL) mainw->pulsed->in_use=TRUE; // audio generator is active
   }
 }
 
