@@ -297,6 +297,9 @@ void on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
       g_free (msg);
     }
     else {
+#ifdef IS_MINGW
+      setmode(fd, O_BINARY);
+#endif
       msg=g_strdup("LiVES filter defaults file version 1.1\n");
       mainw->write_failed=FALSE;
       lives_write(fd,msg,strlen(msg),TRUE);
@@ -328,6 +331,9 @@ void on_save_rte_defs_activate (GtkMenuItem *menuitem, gpointer user_data) {
       g_free (msg);
     }
     else {
+#ifdef IS_MINGW
+      setmode(fd, O_BINARY);
+#endif
       msg=g_strdup("LiVES generator default sizes file version 2\n");
       mainw->write_failed=FALSE;
       lives_write(fd,msg,strlen(msg),TRUE);
@@ -379,6 +385,9 @@ void load_rte_defs (void) {
 	retval=do_read_failed_error_s_with_retry(prefs->fxdefsfile,g_strerror(errno),NULL);
       }
       else {
+#ifdef IS_MINGW
+	setmode(fd, O_BINARY);
+#endif
 	mainw->read_failed=FALSE;
 	msg2=g_strdup_printf(_("Loading real time effect defaults from %s..."),prefs->fxdefsfile);
 	d_print(msg2);
@@ -426,6 +435,9 @@ void load_rte_defs (void) {
 	if (retval==LIVES_CANCEL) return;
       }
       else {
+#ifdef IS_MINGW
+	setmode(fd, O_BINARY);
+#endif
 	msg2=g_strdup_printf(_("Loading generator default sizes from %s..."),prefs->fxsizesfile);
 	d_print(msg2);
 	g_free(msg2);
@@ -558,6 +570,12 @@ gboolean on_load_keymap_clicked (GtkButton *button, gpointer user_data) {
 
     if (keymap_file2!=NULL) {
       if ((kfd=open(keymap_file,O_RDONLY))==-1) has_error=TRUE;
+#ifdef IS_MINGW
+      else {
+	setmode(kfd, O_BINARY);
+      }
+#endif
+
     }
     else {
       if (!(kfile=fopen (keymap_file,"r"))) {
@@ -1104,8 +1122,10 @@ on_rtew_ok_clicked (GtkButton *button, gpointer user_data) {
 
 
 
-void do_mix_error(void) {
-  do_error_dialog_with_check_transient(_("\n\nThis version of LiVES does not allowing mixing of generators and non-generators on the same key.\n\n"),FALSE,0,GTK_WINDOW(rte_window));
+static void do_mix_error(void) {
+  do_error_dialog_with_check_transient(
+				       _("\n\nThis version of LiVES does not allowing mixing of generators and non-generators on the same key.\n\n"),
+				       FALSE,0,GTK_WINDOW(rte_window));
   return;
 }
 
@@ -1721,6 +1741,7 @@ void rte_set_defs_ok (GtkButton *button, lives_rfx_t *rfx) {
 
   on_paramwindow_cancel_clicked(button,rfx);
   fx_dialog[1]=NULL;
+
 }
 
 
@@ -1826,7 +1847,8 @@ void rte_reset_defs_clicked (GtkButton *button, lives_rfx_t *rfx) {
 
 void load_default_keymap(void) {
   // called on startup
-  gchar *keymap_file=g_build_filename(capable->home_dir,LIVES_CONFIG_DIR,"default.keymap",NULL);
+  gchar *dir=g_build_filename(capable->home_dir,LIVES_CONFIG_DIR,NULL);
+  gchar *keymap_file=g_build_filename(dir,"default.keymap",NULL);
   gchar *keymap_template=g_build_filename(prefs->prefix_dir,DATA_DIR,"default.keymap",NULL);
   gchar *com,*tmp;
 
@@ -1837,7 +1859,17 @@ void load_default_keymap(void) {
   do {
     retval=0;
     if (!g_file_test (keymap_file, G_FILE_TEST_EXISTS)) {
+
+      if (!g_file_test(dir,G_FILE_TEST_IS_DIR)) {
+	g_mkdir_with_parents(dir,S_IRWXU);
+      }
+	
+#ifndef IS_MINGW
       com=g_strdup_printf("/bin/cp \"%s\" \"%s\"",keymap_template,keymap_file);
+#else
+      com=g_strdup_printf("cp.exe \"%s\" \"%s\"",keymap_template,keymap_file);
+#endif
+
       lives_system(com,TRUE); // allow this to fail - we will check for errors below
       g_free(com);
     }
@@ -1854,7 +1886,8 @@ void load_default_keymap(void) {
       if (retval==LIVES_CANCEL) {
 	g_free(keymap_file);
 	g_free(keymap_template);
-      
+	g_free(dir);
+
 	threaded_dialog_spin();
 	return;
       }
@@ -1865,5 +1898,6 @@ void load_default_keymap(void) {
 
   g_free(keymap_file);
   g_free(keymap_template);
+  g_free(dir);
   threaded_dialog_spin();
 }
