@@ -62,9 +62,12 @@ gboolean do_tempdir_query(void) {
   gint response;
   gboolean ok=FALSE;
   _entryw *tdentry;
-  gchar *com,*dirname;
+  gchar *dirname;
   guint64 freesp;
 
+#ifndef IS_MINGW
+  gchar *com;
+#endif
 
  top:
 
@@ -137,16 +140,16 @@ gboolean do_tempdir_query(void) {
   g_free(tdentry);
 
   mainw->com_failed=FALSE;
-  com=g_strdup_printf ("/bin/mkdir -p \"%s\" 2>/dev/null",dirname);
+
+  if (!g_file_test(dirname,G_FILE_TEST_IS_DIR)) {
+    if (g_mkdir_with_parents(dirname,S_IRWXU)==-1) goto top;
+  }
+
+#ifndef IS_MINGW
+  com=g_strdup_printf ("/bin/chmod 777 \"%s\" 2>/dev/null",dirname);
   lives_system (com,FALSE);
   g_free (com);
-
-  if (mainw->com_failed) goto top;
-
-
-  com=g_strdup_printf("/bin/rmdir \"%s\" 2>/dev/null",prefs->tmpdir);
-  lives_system(com,TRUE);
-  g_free(com);
+#endif
 
   g_snprintf(prefs->tmpdir,PATH_MAX,"%s",dirname);
   g_snprintf(future_prefs->tmpdir,PATH_MAX,"%s",prefs->tmpdir);
@@ -154,7 +157,11 @@ gboolean do_tempdir_query(void) {
   set_pref("tempdir",prefs->tmpdir);
   set_pref("session_tempdir",prefs->tmpdir);
 
-  g_snprintf(mainw->first_info_file,PATH_MAX,"%s/.info.%d",prefs->tmpdir,getpid());
+#ifndef IS_MINGW
+  g_snprintf(mainw->first_info_file,PATH_MAX,"%s"G_DIR_SEPARATOR_S".info.%d",prefs->tmpdir,getpid());
+#else
+  g_snprintf(mainw->first_info_file,PATH_MAX,"%s"G_DIR_SEPARATOR_S"info.%d",prefs->tmpdir,getpid());
+#endif
 
   g_free(dirname);
   return TRUE;
@@ -234,7 +241,7 @@ gboolean do_audio_choice_dialog(short startup_phase) {
 
   txt5=g_strdup(_("SOX may be used if neither of the preceding players work, "));
 
-  if (capable->has_sox) {
+  if (capable->has_sox_play) {
     txt6=g_strdup(_("but some audio features will be disabled.\n\n"));
   }
   else {
@@ -357,7 +364,7 @@ gboolean do_audio_choice_dialog(short startup_phase) {
 #endif
 
   radiobutton2 = gtk_radio_button_new (NULL);
-  if (capable->has_sox) {
+  if (capable->has_sox_play) {
 
     eventbox=gtk_event_box_new();
     label=gtk_label_new_with_mnemonic ( _("Use _sox audio player"));
@@ -662,7 +669,7 @@ gboolean do_startup_tests(gboolean tshoot) {
   add_test(table,0,_("Checking for \"sox\" presence"),TRUE);
 
 
-  if (!capable->has_sox) {
+  if (!capable->has_sox_sox) {
     success=fail_test(table,0,_("You should install sox to be able to use all the audio features in LiVES"));
 
   }
@@ -678,10 +685,10 @@ gboolean do_startup_tests(gboolean tshoot) {
 
   get_temp_handle(mainw->first_free_file,TRUE);
 
-
   if (success) {
     
     info_fd=-1;
+
     unlink(cfile->info_file);
 
     // write 1 second of silence
@@ -699,6 +706,9 @@ gboolean do_startup_tests(gboolean tshoot) {
 	g_free(tmp);
       }
       else {
+#ifdef IS_MINGW
+	setmode(out_fd, O_BINARY);
+#endif
 	lives_write (out_fd,abuff,176400,TRUE);
 	close(out_fd);
 	g_free(abuff);
@@ -717,7 +727,7 @@ gboolean do_startup_tests(gboolean tshoot) {
       afile=g_build_filename(prefs->tmpdir,cfile->handle,"testout.wav",NULL);
     
       mainw->com_failed=FALSE;
-      com=g_strdup_printf("smogrify export_audio \"%s\" 0. 0. 44100 2 16 0 22050 \"%s\"",cfile->handle,afile);
+      com=g_strdup_printf("%s export_audio \"%s\" 0. 0. 44100 2 16 0 22050 \"%s\"",prefs->backend_sync,cfile->handle,afile);
       lives_system(com,TRUE);
       if (mainw->com_failed) {
 	tmp=g_strdup_printf(_("Command failed: %s"),com);
@@ -726,8 +736,8 @@ gboolean do_startup_tests(gboolean tshoot) {
       }
       
       g_free(com);
-      
-      while (mainw->cancelled==CANCEL_NONE&&(info_fd=open(cfile->info_file,O_RDONLY)==-1)) {
+
+      while (mainw->cancelled==CANCEL_NONE&&(info_fd=open(cfile->info_file,O_RDONLY))==-1) {
 	g_usleep(prefs->sleep_time);
 	while (g_main_context_iteration(NULL,FALSE));
       }
@@ -735,9 +745,10 @@ gboolean do_startup_tests(gboolean tshoot) {
       if (info_fd!=-1) {
 	close(info_fd);
 	
-	sync();
+	lives_sync();
 	
 	fsize=sget_file_size(afile);
+	unlink(afile);
 	g_free(afile);
 	
 	if (fsize==0) {
@@ -745,6 +756,7 @@ gboolean do_startup_tests(gboolean tshoot) {
 	}
 	
 	else pass_test(table,1);
+
       }
     }
   }
@@ -801,9 +813,12 @@ gboolean do_startup_tests(gboolean tshoot) {
   add_test(table,3,_("Checking if mplayer can convert audio"),success2);
 
   if (success2) {
+
+#ifndef IS_MINGW
     res=system("LANG=en LANGUAGE=en mplayer -ao help | grep pcm >/dev/null 2>&1");
-    
-    
+#else    
+    res=system("mplayer -ao help | grep pcm >NUL 2>&1");
+#endif
     if (res==0) {
       pass_test(table,3);
     }
@@ -834,11 +849,12 @@ gboolean do_startup_tests(gboolean tshoot) {
   if (success2&&success4) {
 
     info_fd=-1;
+
     unlink(cfile->info_file);
 
     rname=get_resource("vidtest.avi");
 
-    com=g_strdup_printf("smogrify open_test \"%s\" \"%s\" 0 png",cfile->handle,
+    com=g_strdup_printf("%s open_test \"%s\" \"%s\" 0 png",prefs->backend_sync,cfile->handle,
 			(tmp=g_filename_from_utf8 (rname,-1,NULL,NULL,NULL)));
     g_free(tmp);
     g_free(rname);
@@ -853,14 +869,15 @@ gboolean do_startup_tests(gboolean tshoot) {
 
     g_free(com);
 
-    while (mainw->cancelled==CANCEL_NONE&&(info_fd=open(cfile->info_file,O_RDONLY)==-1)) {
+    while (mainw->cancelled==CANCEL_NONE&&(info_fd=open(cfile->info_file,O_RDONLY))==-1) {
       g_usleep(prefs->sleep_time);
     }
     
     if (info_fd!=-1) {
+
       close(info_fd);
       
-      sync();
+      lives_sync();
       
       cfile->img_type=IMG_TYPE_PNG;
       get_frame_count(mainw->current_file);
@@ -896,7 +913,11 @@ gboolean do_startup_tests(gboolean tshoot) {
 
 
   if (success2) {
+#ifndef IS_MINGW
     res=system("LANG=en LANGUAGE=en mplayer -vo help | grep -i \"jpeg file\" >/dev/null 2>&1");
+#else    
+    res=system("mplayer -vo help | grep -i \"jpeg file\" >NUL 2>&1");
+#endif
     
     if (res==0) {
       pass_test(table,5);
